from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
import unittest
import sys
import os
path = os.path.dirname(os.path.realpath(__file__))
path = "{0}/../../plugins/module_utils".format(path)
sys.path.append(path)
import mongodb_common
from distutils.version import LooseVersion


class FakeAnsibleModule:

    def __init__(self):
        self.msg = ""

    def get_msg(self):
        return self.msg

    def fail_json(self, msg):
        self.msg = msg


class TestMongoDBCommonMethods(unittest.TestCase):

    def test_check_compatibility_old_pymongo_version(self):
        # (mongo version, pymongo version, msg)
        versions = [
          ('2.6', '2.0', 'you must use pymongo 2.7+ with MongoDB 2.6'),
          ('3.0', '2.7', 'you must use pymongo 2.8+ with MongoDB 3.0'),
          ('3.2', '3.0', 'you must use pymongo 3.2+ with MongoDB >= 3.2'),
          ('3.4', '3.3', 'you must use pymongo 3.4+ with MongoDB >= 3.4'),
          ('3.6', '3.5', 'you must use pymongo 3.6+ with MongoDB >= 3.6'),
          ('4.0', '3.6', 'you must use pymongo 3.7+ with MongoDB >= 4.0'),
          ('4.2', '3.8', 'you must use pymongo 3.9+ with MongoDB >= 4.2')
        ]

        for tuple in versions:
            fake_module = FakeAnsibleModule()
            mongodb_common.check_compatibility(fake_module, LooseVersion(tuple[0]), LooseVersion(tuple[1]))
            msg = fake_module.get_msg()
            assert tuple[2] in msg

    def test_check_compatibility_correct_pymongo_version(self):
        # (mongo version, pymongo version)
        versions = [
          ('2.6', '2.8'),
          ('3.0', '2.9'),
          ('3.2', '3.2'),
          ('3.4', '3.4'),
          ('3.6', '3.6'),
          ('4.0', '3.7'),
          ('4.2', '3.9')
        ]

        for tuple in versions:
            fake_module = FakeAnsibleModule()
            mongodb_common.check_compatibility(fake_module, LooseVersion(tuple[0]), LooseVersion(tuple[1]))
            msg = fake_module.get_msg()
            assert msg == ""  # Up-to-date pymongo versions get no message

    def test_load_mongocnf(self):
        with open(os.path.expanduser("~/.mongodb.cnf"), "w+") as w:
            w.write("""
            [client]
            user = mongo_user
            pass = 123456
            """)
        creds = mongodb_common.load_mongocnf()
        assert creds['user'] == "mongo_user"
        assert creds['password'] == "123456"

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


class ModuleDocFragment(object):
    # Parameters for VMware REST Client based modules
    DOCUMENTATION = r'''
options:
  hostname:
    description:
    - The hostname or IP address of the vSphere vCenter server.
    - If the value is not specified in the task, the value of environment variable C(VMWARE_HOST) will be used instead.
    type: str
  username:
    description:
    - The username of the vSphere vCenter server.
    - If the value is not specified in the task, the value of environment variable C(VMWARE_USER) will be used instead.
    type: str
    aliases: [ admin, user ]
  password:
    description:
    - The password of the vSphere vCenter server.
    - If the value is not specified in the task, the value of environment variable C(VMWARE_PASSWORD) will be used instead.
    type: str
    aliases: [ pass, pwd ]
  validate_certs:
    description:
    - Allows connection when SSL certificates are not valid.
    - Set to C(no) when certificates are not trusted.
    - If the value is not specified in the task, the value of environment variable C(VMWARE_VALIDATE_CERTS) will be used instead.
    type: bool
    default: yes
  port:
    description:
    - The port number of the vSphere vCenter.
    - If the value is not specified in the task, the value of environment variable C(VMWARE_PORT) will be used instead.
    type: int
    default: 443
  protocol:
    description:
    - The connection to protocol.
    type: str
    choices: [ http, https ]
    default: https
'''



#!/usr/bin/python

# (c) 2018, Rhys Campbell <rhys.james.campbell@googlemail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: mongodb_shard
version_added: "2.8"
short_description: Add or remove shards from a MongoDB Cluster
description:
    -  Add or remove shards from a MongoDB Cluster.
author: Rhys Campbell (@rhysmeister)
options:
  login_user:
    description:
      - The MongoDB user to login with.
    required: false
    type: str
  login_password:
    description:
      - The login user's password used to authenticate with.
    required: false
    type: str
  login_database:
    description:
      - The database where login credentials are stored.
    required: false
    type: str
    default: admin
  login_host:
    description:
      - The host to login to.
      - This must be a mongos.
    required: false
    type: str
    default: localhost
  login_port:
    description:
      - The MongoDB port to login to.
    required: false
    type: int
    default: 27017
  shard:
    description:
      - The shard connection string.
      - Should be supplied in the form <replicaset>/host:port as detailed in U(https://docs.mongodb.com/manual/tutorial/add-shards-to-shard-cluster/).
      - For example rs0/example1.mongodb.com:27017.
    required: true
    type: str
  sharded_databases:
    description:
      - Enable sharding on the listed database.
      - Can be supplied as a string or a list of strings.
      - Sharding cannot be disabled on a database.
    required: false
    type: raw
  balancer_state:
    description:
      - Manage the Balancer for the Cluster
    required: false
    type: str
    choices: ["started", "stopped", None]
  autosplit:
    description:
      - Disable or enable the autosplit flag in the config.settings collection.
    required: false
    type: bool
  mongos_process:
    description:
      - Provide a custom name for the mongos process you are connecting to.
      - Most users can ignore this setting.
    required: false
    type: str
    default: "mongos"
  ssl:
    description:
      - Whether to use an SSL connection when connecting to the database.
    default: False
    type: bool
  ssl_cert_reqs:
    description:
      - Specifies whether a certificate is required from the other side of the connection, and whether it will be validated if provided.
    required: false
    type: str
    default: CERT_REQUIRED
    choices: [CERT_NONE, CERT_OPTIONAL, CERT_REQUIRED]
  state:
    description:
      - Whether the shard should be present or absent from the Cluster.
    required: false
    type: str
    default: present
    choices: [absent, present]

notes:
    - Requires the pymongo Python package on the remote host, version 2.4.2+.
requirements: [ pymongo ]
'''

EXAMPLES = '''
- name: Add a replicaset shard named rs1 with a member running on port 27018 on mongodb0.example.net
  mongodb_shard:
    login_user: admin
    login_password: admin
    shard: "rs1/mongodb0.example.net:27018"
    state: present

- name: Add a standalone mongod shard running on port 27018 of mongodb0.example.net
  mongodb_shard:
    login_user: admin
    login_password: admin
    shard: "mongodb0.example.net:27018"
    state: present

- name: To remove a shard called 'rs1'
  mongodb_shard:
    login_user: admin
    login_password: admin
    shard: rs1
    state: absent

# Single node shard running on localhost
- name: Ensure shard rs0 exists
  mongodb_shard:
    login_user: admin
    login_password: secret
    shard: "rs0/localhost:3001"
    state: present

# Single node shard running on localhost
- name: Ensure shard rs1 exists
  mongodb_shard:
    login_user: admin
    login_password: secret
    shard: "rs1/localhost:3002"
    state: present

# Enable sharding on a few databases when creating the shard
- name: To remove a shard called 'rs1'
  mongodb_shard:
    login_user: admin
    login_password: admin
    shard: rs1
    sharded_databases:
      - db1
      - db2
    state: present
'''

RETURN = '''
mongodb_shard:
    description: The name of the shard to create.
    returned: success
    type: str
sharded_enabled:
    description: Databases that have had sharding enabled during module execution.
    returned: success when sharding is enabled
    type: list
'''

import os
import ssl as ssl_lib
from distutils.version import LooseVersion
import traceback
import time


from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.six import binary_type, text_type
from ansible.module_utils.six.moves import configparser
from ansible.module_utils._text import to_native
from ansible_collections.community.mongodb.plugins.module_utils.mongodb_common import check_compatibility, missing_required_lib, load_mongocnf
from ansible_collections.community.mongodb.plugins.module_utils.mongodb_common import PyMongoVersion, PYMONGO_IMP_ERR, pymongo_found, MongoClient


def shard_find(client, shard):
    """Check if a shard exists.

    Args:
        client (cursor): Mongodb cursor on admin database.
        shard (str): shard to check.

    Returns:
        dict: when user exists, False otherwise.
    """
    if '/' in shard:
        s = shard.split('/')[0]
    else:
        s = shard
    for shard in client["config"].shards.find({"_id": s}):
        return shard
    return False


def shard_add(client, shard):
    try:
        sh = client["admin"].command('addShard', shard)
    except Exception as excep:
        raise excep
    return sh


def shard_remove(client, shard):
    try:
        sh = client["admin"].command('removeShard', shard)
    except Exception as excep:
        raise excep
    return sh


def sharded_dbs(client):
    '''
    Returns the sharded databases
    Args:
        client (cursor): Mongodb cursor on admin database.
    Returns:
        a list of database names that are sharded
    '''
    sharded_databases = []
    for entry in client["config"].databases.find({"partitioned": True}, {"_id": 1}):
        sharded_databases.append(entry["_id"])
    return sharded_databases


def enable_database_sharding(client, database):
    '''
    Enables sharding on a database
    Args:
        client (cursor): Mongodb cursor on admin database.
    Returns:
        true on success, false on failure
    '''
    s = False
    db = client["admin"].command('enableSharding', database)
    if db:
        s = True
    return s


def any_dbs_to_shard(client, sharded_databases):
    '''
    Return a list of databases that need to have sharding enabled
    sharded_databases - Provided by module
    cluster_sharded_databases - List of sharded dbs from the mongos
    '''
    dbs_to_shard = []
    cluster_sharded_databases = sharded_dbs(client)
    for db in sharded_databases:
        if db not in cluster_sharded_databases:
            dbs_to_shard.append(db)
    return dbs_to_shard


def get_balancer_state(client):
    '''
    Gets the state of the MongoDB balancer. The config.settings collection does
    not exist until the balancer has been started for the first time
    { "_id" : "balancer", "mode" : "full", "stopped" : false }
    { "_id" : "autosplit", "enabled" : true }
    '''
    balancer_state = "stopped"
    result = client["config"].settings.find_one({"_id": "balancer"})
    if not result:
        balancer_state = "stopped"
    else:
        if result['stopped'] is False:
            balancer_state = "started"
        else:
            balancer_state = "stopped"
    return balancer_state


def stop_balancer(client):
    '''
    Stops MongoDB balancer
    '''
    client['admin'].command({'balancerStop': 1, 'maxTimeMS': 60000})
    time.sleep(1)


def start_balancer(client):
    '''
    Starts MongoDB balancer
    '''
    client['admin'].command({'balancerStart': 1, 'maxTimeMS': 60000})
    time.sleep(1)


def enable_autosplit(client):
    client["config"].settings.update({"_id": "autosplit"},
                                     {"$set": {"enabled": True}},
                                     upsert=True,
                                     w="majority")


def disable_autosplit(client):
    client["config"].settings.update({"_id": "autosplit"},
                                     {"$set": {"enabled": False}},
                                     upsert=True,
                                     w="majority")


def get_autosplit(client):
    autosplit = False
    result = client["config"].settings.find_one({"_id": "autosplit"})
    if result is not None:
        autosplit = result['enabled']
    return autosplit


# =========================================
# Module execution.
#


def main():
    module = AnsibleModule(argument_spec=dict(login_user=dict(type='str', required=False),
                                              login_password=dict(type='str', required=False, no_log=True),
                                              login_database=dict(type='str', required=False, default='admin'),
                                              login_host=dict(type='str', required=False, default='localhost'),
                                              login_port=dict(type='int', default=27017, required=False),
                                              ssl=dict(type='bool', default=False, required=False),
                                              ssl_cert_reqs=dict(type='str', required=False, default='CERT_REQUIRED',
                                                                 choices=['CERT_NONE', 'CERT_OPTIONAL', 'CERT_REQUIRED']),
                                              shard=dict(type='str', required=True),
                                              sharded_databases=dict(type="raw", required=False),
                                              balancer_state=dict(type='str', required=False, choices=["started", "stopped", None], default=None),
                                              autosplit=dict(type='bool', required=False, default=None),
                                              mongos_process=dict(type='str', required=False, default="mongos"),
                                              state=dict(type='str', required=False, default='present', choices=['absent', 'present'])),
                           supports_check_mode=True)

    if not pymongo_found:
        module.fail_json(msg=missing_required_lib('pymongo'),
                         exception=PYMONGO_IMP_ERR)

    login_user = module.params['login_user']
    login_password = module.params['login_password']
    login_database = module.params['login_database']
    login_host = module.params['login_host']
    login_port = module.params['login_port']
    ssl = module.params['ssl']
    shard = module.params['shard']
    state = module.params['state']
    sharded_databases = module.params['sharded_databases']
    balancer_state = module.params['balancer_state']
    autosplit = module.params['autosplit']
    mongos_process = module.params['mongos_process']

    try:
        connection_params = {
            "host": login_host,
            "port": int(login_port)
        }

        if ssl:
            connection_params["ssl"] = ssl
            connection_params["ssl_cert_reqs"] = getattr(ssl_lib, module.params['ssl_cert_reqs'])

        client = MongoClient(**connection_params)

        try:
            # Get server version:
            try:
                srv_version = LooseVersion(client.server_info()['version'])
            except Exception as e:
                module.fail_json(msg='Unable to get MongoDB server version: %s' % to_native(e))

            # Get driver version::
            driver_version = LooseVersion(PyMongoVersion)

            # Check driver and server version compatibility:
            check_compatibility(module, srv_version, driver_version)
        except Exception as excep:
            if "not authorized on" in str(excep) or "there are no users authenticated" in str(excep):
                if login_user is not None and login_password is not None:
                    client.admin.authenticate(login_user, login_password, source=login_database)
                    check_compatibility(module, client)
                else:
                    raise excep
            else:
                raise excep

        if login_user is None and login_password is None:
            mongocnf_creds = load_mongocnf()
            if mongocnf_creds is not False:
                login_user = mongocnf_creds['user']
                login_password = mongocnf_creds['password']
        elif login_password is None or login_user is None:
            module.fail_json(msg='when supplying login arguments, both login_user and login_password must be provided')

        try:
            client['admin'].command('listDatabases', 1.0)  # if this throws an error we need to authenticate
        except Exception as excep:
            if "not authorized on" in str(excep) or "command listDatabases requires authentication" in str(excep):
                if login_user is not None and login_password is not None:
                    client.admin.authenticate(login_user, login_password, source=login_database)
                else:
                    raise excep
            else:
                raise excep

    except Exception as e:
        module.fail_json(msg='unable to connect to database: %s' % to_native(e), exception=traceback.format_exc())

    try:
        if client["admin"].command("serverStatus")["process"] != mongos_process:
            module.fail_json(msg="Process running on {0}:{1} is not a {2}".format(login_host, login_port, mongos_process))
        shard_created = False
        dbs_to_shard = []
        cluster_autosplit = None
        old_balancer_state = None
        new_balancer_state = None
        old_autosplit = None
        new_autosplit = None
        if sharded_databases is not None:
            if isinstance(sharded_databases, str):
                sharded_databases = list(sharded_databases)
            dbs_to_shard = any_dbs_to_shard(client, sharded_databases)
        if balancer_state is not None:
            cluster_balancer_state = get_balancer_state(client)
        if autosplit is not None:
            cluster_autosplit = get_autosplit(client)
        if module.check_mode:
            if state == "present":
                changed = False
                if not shard_find(client, shard) or len(dbs_to_shard) > 0:
                    changed = True
                if (balancer_state is not None
                        and balancer_state != cluster_balancer_state):
                    old_balancer_state = cluster_balancer_state
                    new_balancer_state = balancer_state
                    changed = True
                if (autosplit is not None
                        and autosplit != cluster_autosplit):
                    old_autosplit = cluster_autosplit
                    new_autosplit = autosplit
                    changed = True
            elif state == "absent":
                if not shard_find(client, shard):
                    changed = False
                else:
                    changed = True
        else:
            if state == "present":
                if not shard_find(client, shard):
                    shard_add(client, shard)
                    changed = True
                else:
                    changed = False
                if len(dbs_to_shard) > 0:
                    for db in dbs_to_shard:
                        enable_database_sharding(client, db)
                    changed = True
                if balancer_state is not None \
                        and balancer_state != cluster_balancer_state:
                    if balancer_state == "started":
                        start_balancer(client)
                        old_balancer_state = cluster_balancer_state
                        new_balancer_state = get_balancer_state(client)
                        changed = True
                    else:
                        stop_balancer(client)
                        old_balancer_state = cluster_balancer_state
                        new_balancer_state = get_balancer_state(client)
                        changed = True
                if autosplit is not None \
                        and autosplit != cluster_autosplit:
                    if autosplit:
                        enable_autosplit(client)
                        old_autosplit = cluster_autosplit
                        new_autosplit = autosplit
                        changed = True
                    else:
                        disable_autosplit(client)
                        old_autosplit = cluster_autosplit
                        new_autosplit = autosplit
                        changed = True
            elif state == "absent":
                if shard_find(client, shard):
                    shard_remove(client, shard)
                    changed = True
                else:
                    changed = False
    except Exception as e:
        action = "add"
        if state == "absent":
            action = "remove"
        module.fail_json(msg='Unable to {0} shard: %s'.format(action) % to_native(e), exception=traceback.format_exc())

    result = {
        "changed": changed,
        "shard": shard,
    }
    if len(dbs_to_shard) > 0:
        result['sharded_enabled'] = dbs_to_shard
    if old_balancer_state is not None:
        result['old_balancer_state'] = old_balancer_state
        result['new_balancer_state'] = new_balancer_state
    if old_autosplit is not None:
        result['old_autosplit'] = old_autosplit
        result['new_autosplit'] = new_autosplit

    module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# (c) 2019, Felix Fontein <felix@fontein.de>
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import, division, print_function
__metaclass__ = type


PEM_START = '-----BEGIN '
PEM_END = '-----'
PKCS8_PRIVATEKEY_NAMES = ('PRIVATE KEY', 'ENCRYPTED PRIVATE KEY')
PKCS1_PRIVATEKEY_SUFFIX = ' PRIVATE KEY'


def identify_pem_format(content):
    '''Given the contents of a binary file, tests whether this could be a PEM file.'''
    try:
        lines = content.decode('utf-8').splitlines(False)
        if lines[0].startswith(PEM_START) and lines[0].endswith(PEM_END) and len(lines[0]) > len(PEM_START) + len(PEM_END):
            return True
    except UnicodeDecodeError:
        pass
    return False


def identify_private_key_format(content):
    '''Given the contents of a private key file, identifies its format.'''
    # See https://github.com/openssl/openssl/blob/master/crypto/pem/pem_pkey.c#L40-L85
    # (PEM_read_bio_PrivateKey)
    # and https://github.com/openssl/openssl/blob/master/include/openssl/pem.h#L46-L47
    # (PEM_STRING_PKCS8, PEM_STRING_PKCS8INF)
    try:
        lines = content.decode('utf-8').splitlines(False)
        if lines[0].startswith(PEM_START) and lines[0].endswith(PEM_END) and len(lines[0]) > len(PEM_START) + len(PEM_END):
            name = lines[0][len(PEM_START):-len(PEM_END)]
            if name in PKCS8_PRIVATEKEY_NAMES:
                return 'pkcs8'
            if len(name) > len(PKCS1_PRIVATEKEY_SUFFIX) and name.endswith(PKCS1_PRIVATEKEY_SUFFIX):
                return 'pkcs1'
            return 'unknown-pem'
    except UnicodeDecodeError:
        pass
    return 'raw'



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The exos_l2_interfaces class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""
from __future__ import absolute_import, division, print_function
__metaclass__ = type

import json
from copy import deepcopy
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import ConfigBase
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list, dict_diff
from ansible_collections.community.network.plugins.module_utils.network.exos.facts.facts import Facts
from ansible_collections.community.network.plugins.module_utils.network.exos.exos import send_requests


class L2_interfaces(ConfigBase):
    """
    The exos_l2_interfaces class
    """

    gather_subset = [
        '!all',
        '!min',
    ]

    gather_network_resources = [
        'l2_interfaces',
    ]

    L2_INTERFACE_NATIVE = {
        "data": {
            "openconfig-vlan:config": {
                "interface-mode": "TRUNK",
                "native-vlan": None,
                "trunk-vlans": []
            }
        },
        "method": "PATCH",
        "path": None
    }

    L2_INTERFACE_TRUNK = {
        "data": {
            "openconfig-vlan:config": {
                "interface-mode": "TRUNK",
                "trunk-vlans": []
            }
        },
        "method": "PATCH",
        "path": None
    }

    L2_INTERFACE_ACCESS = {
        "data": {
            "openconfig-vlan:config": {
                "interface-mode": "ACCESS",
                "access-vlan": None
            }
        },
        "method": "PATCH",
        "path": None
    }

    L2_PATH = "/rest/restconf/data/openconfig-interfaces:interfaces/interface="

    def __init__(self, module):
        super(L2_interfaces, self).__init__(module)

    def get_l2_interfaces_facts(self):
        """ Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset, self.gather_network_resources)
        l2_interfaces_facts = facts['ansible_network_resources'].get(
            'l2_interfaces')
        if not l2_interfaces_facts:
            return []
        return l2_interfaces_facts

    def execute_module(self):
        """ Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {'changed': False}
        warnings = list()
        requests = list()

        existing_l2_interfaces_facts = self.get_l2_interfaces_facts()
        requests.extend(self.set_config(existing_l2_interfaces_facts))
        if requests:
            if not self._module.check_mode:
                send_requests(self._module, requests=requests)
            result['changed'] = True
        result['requests'] = requests

        changed_l2_interfaces_facts = self.get_l2_interfaces_facts()

        result['before'] = existing_l2_interfaces_facts
        if result['changed']:
            result['after'] = changed_l2_interfaces_facts

        result['warnings'] = warnings
        return result

    def set_config(self, existing_l2_interfaces_facts):
        """ Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the requests necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params['config']
        have = existing_l2_interfaces_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """ Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the requests necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params['state']
        if state == 'overridden':
            requests = self._state_overridden(want, have)
        elif state == 'deleted':
            requests = self._state_deleted(want, have)
        elif state == 'merged':
            requests = self._state_merged(want, have)
        elif state == 'replaced':
            requests = self._state_replaced(want, have)
        return requests

    def _state_replaced(self, want, have):
        """ The request generator when state is replaced

        :rtype: A list
        :returns: the requests necessary to migrate the current configuration
                  to the desired configuration
        """
        requests = []
        for w in want:
            for h in have:
                if w["name"] == h["name"]:
                    if dict_diff(w, h):
                        l2_request = self._update_patch_request(w, h)
                        l2_request["data"] = json.dumps(l2_request["data"])
                        requests.append(l2_request)
                    break

        return requests

    def _state_overridden(self, want, have):
        """ The request generator when state is overridden

        :rtype: A list
        :returns: the requests necessary to migrate the current configuration
                  to the desired configuration
        """
        requests = []
        have_copy = []
        for w in want:
            for h in have:
                if w["name"] == h["name"]:
                    if dict_diff(w, h):
                        l2_request = self._update_patch_request(w, h)
                        l2_request["data"] = json.dumps(l2_request["data"])
                        requests.append(l2_request)
                    have_copy.append(h)
                    break

        for h in have:
            if h not in have_copy:
                l2_delete = self._update_delete_request(h)
                if l2_delete["path"]:
                    l2_delete["data"] = json.dumps(l2_delete["data"])
                    requests.append(l2_delete)

        return requests

    def _state_merged(self, want, have):
        """ The request generator when state is merged

        :rtype: A list
        :returns: the requests necessary to merge the provided into
                  the current configuration
        """
        requests = []
        for w in want:
            for h in have:
                if w["name"] == h["name"]:
                    if dict_diff(h, w):
                        l2_request = self._update_patch_request(w, h)
                        l2_request["data"] = json.dumps(l2_request["data"])
                        requests.append(l2_request)
                    break

        return requests

    def _state_deleted(self, want, have):
        """ The request generator when state is deleted

        :rtype: A list
        :returns: the requests necessary to remove the current configuration
                  of the provided objects
        """
        requests = []
        if want:
            for w in want:
                for h in have:
                    if w["name"] == h["name"]:
                        l2_delete = self._update_delete_request(h)
                        if l2_delete["path"]:
                            l2_delete["data"] = json.dumps(l2_delete["data"])
                            requests.append(l2_delete)
                        break

        else:
            for h in have:
                l2_delete = self._update_delete_request(h)
                if l2_delete["path"]:
                    l2_delete["data"] = json.dumps(l2_delete["data"])
                    requests.append(l2_delete)

        return requests

    def _update_patch_request(self, want, have):

        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset, ['vlans', ])
        vlans_facts = facts['ansible_network_resources'].get('vlans')

        vlan_id = []

        for vlan in vlans_facts:
            vlan_id.append(vlan['vlan_id'])

        if want.get("access"):
            if want["access"]["vlan"] in vlan_id:
                l2_request = deepcopy(self.L2_INTERFACE_ACCESS)
                l2_request["data"]["openconfig-vlan:config"]["access-vlan"] = want["access"]["vlan"]
                l2_request["path"] = self.L2_PATH + str(want["name"]) + "/openconfig-if-ethernet:ethernet/openconfig-vlan:switched-vlan/config"
            else:
                self._module.fail_json(msg="VLAN %s does not exist" % (want["access"]["vlan"]))

        elif want.get("trunk"):
            if want["trunk"]["native_vlan"]:
                if want["trunk"]["native_vlan"] in vlan_id:
                    l2_request = deepcopy(self.L2_INTERFACE_NATIVE)
                    l2_request["data"]["openconfig-vlan:config"]["native-vlan"] = want["trunk"]["native_vlan"]
                    l2_request["path"] = self.L2_PATH + str(want["name"]) + "/openconfig-if-ethernet:ethernet/openconfig-vlan:switched-vlan/config"
                    for vlan in want["trunk"]["trunk_allowed_vlans"]:
                        if int(vlan) in vlan_id:
                            l2_request["data"]["openconfig-vlan:config"]["trunk-vlans"].append(int(vlan))
                        else:
                            self._module.fail_json(msg="VLAN %s does not exist" % (vlan))
                else:
                    self._module.fail_json(msg="VLAN %s does not exist" % (want["trunk"]["native_vlan"]))
            else:
                l2_request = deepcopy(self.L2_INTERFACE_TRUNK)
                l2_request["path"] = self.L2_PATH + str(want["name"]) + "/openconfig-if-ethernet:ethernet/openconfig-vlan:switched-vlan/config"
                for vlan in want["trunk"]["trunk_allowed_vlans"]:
                    if int(vlan) in vlan_id:
                        l2_request["data"]["openconfig-vlan:config"]["trunk-vlans"].append(int(vlan))
                    else:
                        self._module.fail_json(msg="VLAN %s does not exist" % (vlan))
        return l2_request

    def _update_delete_request(self, have):

        l2_request = deepcopy(self.L2_INTERFACE_ACCESS)

        if have["access"] and have["access"]["vlan"] != 1 or have["trunk"] or not have["access"]:
            l2_request["data"]["openconfig-vlan:config"]["access-vlan"] = 1
            l2_request["path"] = self.L2_PATH + str(have["name"]) + "/openconfig-if-ethernet:ethernet/openconfig-vlan:switched-vlan/config"

        return l2_request



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_snmp_hosts
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxSNMPHostsModule(TestOnyxModule):

    module = onyx_snmp_hosts

    def setUp(self):
        self.enabled = False
        super(TestOnyxSNMPHostsModule, self).setUp()
        self.mock_get_config = patch.object(
            onyx_snmp_hosts.OnyxSNMPHostsModule, "_show_hosts_config")
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestOnyxSNMPHostsModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        config_file = 'onyx_show_snmp_hosts.cfg'
        self.get_config.return_value = load_fixture(config_file)
        self.load_config.return_value = None

    def test_snmp_host_enabled_state_no_change(self):
        set_module_args(dict(hosts=[dict(name='1.1.1.1',
                                         enabled=True)]))
        self.execute_module(changed=False)

    def test_snmp_host_enabled_state_with_change(self):
        set_module_args(dict(hosts=[dict(name='1.1.1.1',
                                         enabled=False)]))
        commands = ['snmp-server host 1.1.1.1 disable']
        self.execute_module(changed=True, commands=commands)

    def test_snmp_host_notification_type_no_change(self):
        set_module_args(dict(hosts=[dict(name='2.2.2.2',
                                         notification_type='trap',
                                         version='2c',
                                         port='5')]))
        self.execute_module(changed=False)

    def test_snmp_host_notification_type_with_change(self):
        set_module_args(dict(hosts=[dict(name='2.2.2.2',
                                         notification_type='inform',
                                         version='2c',
                                         port='5')]))
        commands = ['snmp-server host 2.2.2.2 informs port 5 version 2c']
        self.execute_module(changed=True, commands=commands)

    def test_snmp_host_version_no_change(self):
        set_module_args(dict(hosts=[dict(name='2.2.2.2',
                                         notification_type='trap',
                                         version='2c',
                                         port='5')]))
        self.execute_module(changed=False)

    def test_snmp_host_version_with_change(self):
        set_module_args(dict(hosts=[dict(name='2.2.2.2',
                                         notification_type='trap',
                                         version='1',
                                         port='5')]))
        commands = ['snmp-server host 2.2.2.2 traps port 5 version 1']
        self.execute_module(changed=True, commands=commands)

    def test_snmp_host_port_no_change(self):
        set_module_args(dict(hosts=[dict(name='2.2.2.2',
                                         notification_type='trap',
                                         version='2c',
                                         port='5')]))
        self.execute_module(changed=False)

    def test_snmp_host_port_with_change(self):
        set_module_args(dict(hosts=[dict(name='2.2.2.2',
                                         notification_type='trap',
                                         version='2c',
                                         port='3')]))
        commands = ['snmp-server host 2.2.2.2 traps port 3 version 2c']
        self.execute_module(changed=True, commands=commands)

    def test_snmp_host_user_name_no_change(self):
        set_module_args(dict(hosts=[dict(name='1.1.1.1',
                                         notification_type='inform',
                                         version='3',
                                         port='3',
                                         user_name='sara',
                                         auth_type='md5',
                                         auth_password='sara123saea1234678')]))
        self.execute_module(changed=False)

    def test_snmp_host_user_name_with_change(self):
        set_module_args(dict(hosts=[dict(name='1.1.1.1',
                                         notification_type='inform',
                                         version='3',
                                         port='3',
                                         user_name='masa',
                                         auth_type='md5',
                                         auth_password='sara123saea1234678')]))
        commands = ['snmp-server host 1.1.1.1 informs port 3 version 3 user masa auth md5 sara123saea1234678']
        self.execute_module(changed=True, commands=commands)

    def test_snmp_host_auth_type_no_change(self):
        set_module_args(dict(hosts=[dict(name='1.1.1.1',
                                         notification_type='inform',
                                         version='3',
                                         port='3',
                                         user_name='sara',
                                         auth_type='md5',
                                         auth_password='sara123saea1234678')]))
        self.execute_module(changed=False)

    def test_snmp_host_auth_type_with_change(self):
        set_module_args(dict(hosts=[dict(name='1.1.1.1',
                                         notification_type='inform',
                                         version='3',
                                         port='3',
                                         user_name='sara',
                                         auth_type='sha',
                                         auth_password='sara123saea1234678')]))
        commands = ['snmp-server host 1.1.1.1 informs port 3 version 3 user sara auth sha sara123saea1234678']
        self.execute_module(changed=True, commands=commands)

    def test_snmp_host_privacy_type_no_change(self):
        set_module_args(dict(hosts=[dict(name='1.1.1.1',
                                         notification_type='inform',
                                         version='3',
                                         port='3',
                                         user_name='sara',
                                         auth_type='md5',
                                         auth_password='sara123saea1234678',
                                         privacy_type='3des',
                                         privacy_password='pjqriuewjhksjmdoiws')]))
        self.execute_module(changed=False)

    def test_snmp_host_privacy_type_with_change(self):
        set_module_args(dict(hosts=[dict(name='1.1.1.1',
                                         notification_type='inform',
                                         version='3',
                                         port='3',
                                         user_name='sara',
                                         auth_type='md5',
                                         auth_password='sara123saea1234678',
                                         privacy_type='aes-192',
                                         privacy_password='pjqriuewjhksjmdoiws')]))
        commands = ['snmp-server host 1.1.1.1 informs port 3 version 3 user sara auth md5 sara123saea1234678 priv aes-192 pjqriuewjhksjmdoiws']
        self.execute_module(changed=True, commands=commands)

    def test_snmp_host_state_with_change(self):
        set_module_args(dict(hosts=[dict(name='2.2.2.2',
                                         notification_type='trap',
                                         version='2c',
                                         port='5',
                                         state='absent')]))
        commands = ['no snmp-server host 2.2.2.2']
        self.execute_module(changed=True, commands=commands)



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_traffic_class
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxTrafficClassModule(TestOnyxModule):

    module = onyx_traffic_class
    arp_suppression = True

    def setUp(self):
        super(TestOnyxTrafficClassModule, self).setUp()
        self.mock_get_congestion_control_config = patch.object(
            onyx_traffic_class.OnyxTrafficClassModule, "_show_interface_congestion_control")
        self.get_congestion_control_config = self.mock_get_congestion_control_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

        self.mock_get_dcb_config = patch.object(
            onyx_traffic_class.OnyxTrafficClassModule, "_show_interface_dcb_ets")
        self.get_dcb_config = self.mock_get_dcb_config.start()

    def tearDown(self):
        super(TestOnyxTrafficClassModule, self).tearDown()
        self.mock_get_congestion_control_config.stop()
        self.mock_load_config.stop()
        self.mock_get_dcb_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        interfaces_congestion_control_config_file = 'onyx_show_interface_congestion_control.cfg'
        interfaces_dcb_config_file = 'onyx_show_dcb_ets_interface.cfg'
        interfaces_congestion_control_data = load_fixture(interfaces_congestion_control_config_file)
        interfaces_dcb_config_data = load_fixture(interfaces_dcb_config_file)
        self.get_congestion_control_config.return_value = interfaces_congestion_control_data
        self.get_dcb_config.return_value = interfaces_dcb_config_data
        self.load_config.return_value = None

    def test_configure_congestion_control_disabled_with_change(self):
        set_module_args(dict(interfaces=["Eth1/1"], tc=1,
                             congestion_control=dict(control="ecn", threshold_mode="absolute",
                                                     min_threshold=500, max_threshold=1500)))
        commands = [
            "interface ethernet 1/1 traffic-class 1 congestion-control ecn minimum-absolute 500 maximum-absolute 1500"
        ]
        self.execute_module(changed=True, commands=commands)

    def test_configure_congestion_control_disabled_with_no_change(self):
        set_module_args(dict(state="disabled", interfaces=["Eth1/1"], tc=0))

        self.execute_module(changed=False)

    def test_configure_congestion_control_with_change(self):
        set_module_args(dict(interfaces=["Eth1/1"], tc=2,
                             congestion_control=dict(control="ecn", threshold_mode="relative",
                                                     min_threshold=9, max_threshold=88)))
        commands = [
            "interface ethernet 1/1 traffic-class 2 congestion-control ecn minimum-relative 9 maximum-relative 88"
        ]
        self.execute_module(changed=True, commands=commands)

    def test_configure_congestion_control_absolute_with_change(self):
        set_module_args(dict(interfaces=["Eth1/1"], tc=3,
                             congestion_control=dict(control="ecn", threshold_mode="absolute",
                                                     min_threshold=500, max_threshold=1500)))
        commands = [
            "interface ethernet 1/1 traffic-class 3 congestion-control ecn minimum-absolute 500 maximum-absolute 1500"
        ]
        self.execute_module(changed=True, commands=commands)

    def test_configure_congestion_control_with_no_change(self):
        set_module_args(dict(interfaces=["Eth1/1"], tc=3,
                             congestion_control=dict(control="ecn", threshold_mode="absolute",
                                                     min_threshold=500, max_threshold=1550)))
        self.execute_module(changed=False)

    def test_configure_dcb_mode_with_no_change(self):
        set_module_args(dict(interfaces=["Eth1/1"], tc=3, dcb=dict(mode="strict")))
        self.execute_module(changed=False)

    def test_configure_dcb_strict_mode_with_change(self):
        set_module_args(dict(interfaces=["Eth1/1"], tc=1, dcb=dict(mode="strict")))
        commands = [
            "interface ethernet 1/1 traffic-class 1 dcb ets strict"
        ]
        self.execute_module(changed=True, commands=commands)

    def test_configure_dcb_wrr_mode_with_change(self):
        set_module_args(dict(interfaces=["Eth1/1"], tc=0, dcb=dict(mode="wrr", weight=10)))
        commands = [
            "interface ethernet 1/1 traffic-class 0 dcb ets wrr 10"
        ]
        self.execute_module(changed=True, commands=commands)

    def test_configure_dcb_wrr_mode_with_no_change(self):
        set_module_args(dict(interfaces=["Eth1/1"], tc=0, dcb=dict(mode="wrr", weight=12)))

        self.execute_module(changed=False)



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_aaa
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxAAAModule(TestOnyxModule):

    module = onyx_aaa

    def setUp(self):
        self.enabled = False
        super(TestOnyxAAAModule, self).setUp()
        self.mock_get_config = patch.object(
            onyx_aaa.OnyxAAAModule, "_show_aaa_config")
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestOnyxAAAModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        config_file = 'onyx_show_aaa.cfg'
        self.get_config.return_value = load_fixture(config_file)
        self.load_config.return_value = None

    def test_aaa_accounting_no_change(self):
        set_module_args(dict(tacacs_accounting_enabled=False))
        self.execute_module(changed=False)

    def test_aaa_accounting_with_change(self):
        set_module_args(dict(tacacs_accounting_enabled=True))
        commands = ['aaa accounting changes default stop-only tacacs+']
        self.execute_module(changed=True, commands=commands)

    def test_aaa_auth_default_user_no_change(self):
        set_module_args(dict(auth_default_user='admin'))
        self.execute_module(changed=False)

    def test_aaa_auth_default_user_with_change(self):
        set_module_args(dict(auth_default_user='monitor'))
        commands = ['aaa authorization map default-user monitor']
        self.execute_module(changed=True, commands=commands)

    def test_aaa_auth_order_no_change(self):
        set_module_args(dict(auth_order='remote-first'))
        self.execute_module(changed=False)

    def test_aaa_auth_order_with_change(self):
        set_module_args(dict(auth_order='local-only'))
        commands = ['aaa authorization map order local-only']
        self.execute_module(changed=True, commands=commands)

    def test_aaa_fallback_no_change(self):
        set_module_args(dict(auth_fallback_enabled=True))
        self.execute_module(changed=False)

    def test_aaa_fallback_with_change(self):
        set_module_args(dict(auth_fallback_enabled=False))
        commands = ['no aaa authorization map fallback server-err']
        self.execute_module(changed=True, commands=commands)



#!/usr/bin/python
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: onyx_snmp_users
version_added: '0.2.0'
author: "Sara Touqan (@sarato)"
short_description: Configures SNMP User parameters
description:
  - This module provides declarative management of SNMP Users protocol params
    on Mellanox ONYX network devices.
options:
  users:
    type: list
    description:
      - List of snmp users
    suboptions:
      name:
        description:
          - Specifies the name of the user.
        required: true
        type: str
      enabled:
        description:
          - Enables/Disables SNMP v3 access for the user.
        type: bool
      set_access_enabled:
        description:
          - Enables/Disables SNMP SET requests for the user.
        type: bool
      require_privacy:
        description:
             - Enables/Disables the Require privacy (encryption) for requests from this user
        type: bool
      auth_type:
        description:
             -  Configures the hash type used to configure SNMP v3 security parameters.
        choices: ['md5', 'sha', 'sha224', 'sha256', 'sha384', 'sha512']
        type: str
      auth_password:
        description:
             - The password needed to configure the hash type.
        type: str
      capability_level:
        description:
             - Sets capability level for SET requests.
        choices: ['admin','monitor','unpriv','v_admin']
        type: str
'''

EXAMPLES = """
- name: Enables snmp user
  onyx_snmp_users:
    users:
       - name: sara
         enabled: true

- name: Enables snmp set requests
  onyx_snmp_users:
    users:
       - name: sara
         set_access_enabled: yes

- name: Enables user require privacy
  onyx_snmp_users:
    users:
       - name: sara
         require_privacy: true

- name: Configures user hash type
  onyx_snmp_users:
    users:
       - auth_type: md5
         auth_password: 1297sara1234sara

- name: Configures user capability_level
  onyx_snmp_users:
    users:
        - name: sara
          capability_level: admin
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device.
  returned: always
  type: list
  sample:
    - snmp-server user <user_name> v3 enable
    - no snmp-server user <user_name> v3 enable
    - snmp-server user <user_name> v3 enable sets
    - no snmp-server user <user_name> v3 enable sets
    - snmp-server user <user_name> v3 require-privacy
    - no snmp-server user <user_name> v3 require-privacy
    - snmp-server user <user_name> v3 capability <capability_level>
    - snmp-server user <user_name> v3 auth <hash_type> <password>
"""

import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.network.plugins.module_utils.network.onyx.onyx import show_cmd
from ansible_collections.community.network.plugins.module_utils.network.onyx.onyx import BaseOnyxModule


class OnyxSNMPUsersModule(BaseOnyxModule):

    def init_module(self):
        """ initialize module
        """
        user_spec = dict(name=dict(required=True),
                         enabled=dict(type='bool'),
                         set_access_enabled=dict(type='bool'),
                         require_privacy=dict(type='bool'),
                         auth_type=dict(type='str', choices=['md5', 'sha', 'sha224', 'sha256', 'sha384', 'sha512']),
                         auth_password=dict(type='str'),
                         capability_level=dict(type='str', choices=['admin', 'monitor', 'unpriv', 'v_admin']),
                         )
        element_spec = dict(
            users=dict(type='list', elements='dict', options=user_spec),
        )
        argument_spec = dict()
        argument_spec.update(element_spec)
        self._module = AnsibleModule(
            argument_spec=argument_spec,
            supports_check_mode=True)

    def get_required_config(self):
        module_params = self._module.params
        self._required_config = dict(module_params)
        self.validate_param_values(self._required_config)

    def _set_snmp_config(self, users_config):
        if users_config[0]:
            if users_config[0].get('Lines'):
                return
        current_users = []
        count = 0
        enabled = True
        set_access_enabled = True
        require_privacy = True
        auth_type = ''
        capability_level = ''
        name = ''
        all_users_names = []
        for user in users_config:
            user_dict = {}
            entry_dict = {}
            for entry in user:
                name = entry.split()[2]
                if user.get(entry):
                    if user.get(entry)[0]:
                        enabled = user.get(entry)[0].get('Enabled overall')
                        if enabled == 'no':
                            enabled = False
                        else:
                            enabled = True
                        set_access_enabled = user.get(entry)[1].get('SET access')[0].get('Enabled')
                        if set_access_enabled == 'no':
                            set_access_enabled = False
                        else:
                            set_access_enabled = True
                        require_privacy = user.get(entry)[0].get('Require privacy')
                        if require_privacy == 'yes':
                            require_privacy = True
                        else:
                            require_privacy = False
                        capability_level = user.get(entry)[1].get('SET access')[0].get('Capability level')
                        auth_type = user.get(entry)[0].get('Authentication type')
                        user_dict['enabled'] = enabled
                        user_dict['set_access_enabled'] = set_access_enabled
                        user_dict['auth_type'] = auth_type
                        user_dict['capability_level'] = capability_level
                        user_dict['require_privacy'] = require_privacy
                        entry_dict[name] = user_dict
                        all_users_names.append(name)
            current_users.append(entry_dict)
        self._current_config['users'] = current_users
        self._current_config['current_names'] = all_users_names

    def _show_users(self):
        cmd = "show snmp user"
        return show_cmd(self._module, cmd, json_fmt=True, fail_on_error=False)

    def load_current_config(self):
        self._current_config = dict()
        users_config = self._show_users()
        if users_config:
            self._set_snmp_config(users_config)

    def generate_commands(self):
        req_uers = self._required_config.get("users")
        current_users = self._current_config.get("users")
        current_names = self._current_config.get("current_names")
        if req_uers:
            for user in req_uers:
                user_id = user.get('name')
                if user_id:
                    if current_names and (user_id in current_names):
                        for user_entry in current_users:
                            for user_name in user_entry:
                                if user_name == user_id:
                                    user_state = user.get("enabled")
                                    user_entry_name = user_entry.get(user_name)
                                    if user_state is not None:
                                        if user_state != user_entry_name.get("enabled"):
                                            if user_state is True:
                                                self._commands.append('snmp-server user {0} v3 enable' .format(user_id))
                                            else:
                                                self._commands.append('no snmp-server user {0} v3 enable' .format(user_id))
                                    set_state = user.get("set_access_enabled")
                                    if set_state is not None:
                                        if set_state != user_entry_name.get("set_access_enabled"):
                                            if set_state is True:
                                                self._commands.append('snmp-server user {0} v3 enable sets' .format(user_id))
                                            else:
                                                self._commands.append('no snmp-server user {0} v3 enable sets' .format(user_id))
                                    auth_type = user.get("auth_type")
                                    if auth_type is not None:
                                        if user.get("auth_password") is not None:
                                            if auth_type != user_entry_name.get("auth_type"):
                                                self._commands.append('snmp-server user {0} v3 auth {1} {2}'
                                                                      .format(user_id, user.get('auth_type'), user.get('auth_password')))
                                    cap_level = user.get("capability_level")
                                    if cap_level is not None:
                                        if cap_level != user_entry_name.get("capability_level"):
                                            self._commands.append('snmp-server user {0} v3 capability {1}'
                                                                  .format(user_id, user.get('capability_level')))
                                    req_priv = user.get("require_privacy")
                                    if req_priv is not None:
                                        if req_priv != user_entry_name.get("require_privacy"):
                                            if req_priv is True:
                                                self._commands.append('snmp-server user {0} v3 require-privacy' .format(user_id))
                                            else:
                                                self._commands.append('no snmp-server user {0} v3 require-privacy' .format(user_id))

                    else:
                        user_state = user.get("enabled")
                        if user_state is not None:
                            if user_state is True:
                                self._commands.append('snmp-server user {0} v3 enable' .format(user_id))
                            else:
                                self._commands.append('no snmp-server user {0} v3 enable' .format(user_id))
                        set_state = user.get("set_access_enabled")
                        if set_state is not None:
                            if set_state is True:
                                self._commands.append('snmp-server user {0} v3 enable sets' .format(user_id))
                            else:
                                self._commands.append('no snmp-server user {0} v3 enable sets' .format(user_id))
                        if user.get("capability_level") is not None:
                            self._commands.append('snmp-server user {0} v3 capability {1}' .format(user_id, user.get('capability_level')))
                        req_priv = user.get("require_privacy")
                        if req_priv is not None:
                            if req_priv is True:
                                self._commands.append('snmp-server user {0} v3 require-privacy' .format(user_id))
                            else:
                                self._commands.append('no snmp-server user {0} v3 require-privacy' .format(user_id))


def main():
    """ main entry point for module execution
    """
    OnyxSNMPUsersModule.main()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, VMware, Inc.
# Copyright: (c) 2019, Abhijeet Kasurde <akasurde@redhat.com>
# SPDX-License-Identifier: GPL-3.0-or-later
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: vmware_dvswitch_nioc
short_description: Manage distributed switch Network IO Control
description:
    - This module can be used to manage distributed switch Network IO Control configurations.
author:
    - Joseph Andreatta (@vmwjoseph)
notes:
    - Tested on vSphere 6.7
requirements:
    - "python >= 2.6"
    - PyVmomi
options:
    switch:
        description:
        - The name of the distributed switch.
        required: True
        aliases: ['dvswitch']
        type: str
    version:
        description:
        - Network IO control version.
        choices:
        - 'version2'
        - 'version3'
        required: False
        type: str
    state:
        description:
        - Enable or disable NIOC on the distributed switch.
        default: 'present'
        choices: ['present', 'absent']
        required: False
        type: str
    resources:
        description:
        - 'List of dicts containing
           { name: Resource name is one of the following: "faultTolerance", "hbr", "iSCSI", "management", "nfs", "vdp",
                   "virtualMachine", "vmotion", "vsan"
             limit: The maximum allowed usage for a traffic class belonging to this resource pool per host physical NIC.
                    reservation: (Ignored if NIOC version is set to version2) Amount of bandwidth resource that is
                    guaranteed available to the host infrastructure traffic class. If the utilization is less than the
                    reservation, the extra bandwidth is used for other host infrastructure traffic class types.
                    Reservation is not allowed to exceed the value of limit, if limit is set. Unit is Mbits/sec.
             shares_level: The allocation level ("low", "normal", "high", "custom"). The level is a simplified view
                           of shares. Levels map to a pre-determined set of numeric values for shares.
             shares: Ignored unless shares_level is "custom".  The number of shares allocated.
             reservation: Ignored unless version is "version3". Amount of bandwidth resource that is guaranteed
                          available to the host infrastructure traffic class.
           }'
        required: False
        type: list
extends_documentation_fragment:
- community.vmware.vmware.documentation

'''

RETURN = r'''
dvswitch_nioc_status:
    description:
    - result of the changes
    returned: success
    type: str
resources_changed:
    description:
    - list of resources which were changed
    returned: success
    type: list
    sample: [ "vmotion", "vsan" ]
'''

EXAMPLES = '''
- name: Enable NIOC
  community.vmware.vmware_dvswitch_nioc:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    switch: dvSwitch
    version: version3
    resources:
        - name: vmotion
          limit: -1
          reservation: 128
          shares_level: normal
        - name: vsan
          limit: -1
          shares_level: custom
          shares: 99
          reservation: 256
    state: present
  delegate_to: localhost

- name: Disable NIOC
  community.vmware.vmware_dvswitch_nioc:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    switch: dvSwitch
    state: absent
  delegate_to: localhost
'''

try:
    from pyVmomi import vim, vmodl
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
from ansible_collections.community.vmware.plugins.module_utils.vmware import (
    PyVmomi,
    find_dvs_by_name,
    vmware_argument_spec,
    wait_for_task)


class VMwareDVSwitchNIOC(PyVmomi):

    def __init__(self, module):
        super(VMwareDVSwitchNIOC, self).__init__(module)
        self.dvs = None
        self.resource_changes = list()
        self.switch = module.params['switch']
        self.version = module.params.get('version')
        self.state = module.params['state']
        self.resources = module.params.get('resources')
        self.result = {
            'changed': False,
            'dvswitch_nioc_status': 'Unchanged',
            'resources_changed': list(),
        }

    def process_state(self):
        nioc_states = {
            'absent': {
                'present': self.state_disable_nioc,
                'absent': self.state_exit,
            },
            'present': {
                'version': self.state_update_nioc_version,
                'update': self.state_update_nioc_resources,
                'present': self.state_exit,
                'absent': self.state_enable_nioc,
            }
        }
        nioc_states[self.state][self.check_nioc_state()]()
        self.state_exit()

    def state_exit(self):
        self.module.exit_json(**self.result)

    def state_disable_nioc(self):
        self.result['changed'] = True
        if not self.module.check_mode:
            self.set_nioc_enabled(False)
        self.result['dvswitch_nioc_status'] = 'Disabled NIOC'

    def state_enable_nioc(self):
        self.result['changed'] = True
        if not self.module.check_mode:
            self.set_nioc_enabled(True)
            self.set_nioc_version()
            self.result['dvswitch_nioc_status'] = "Enabled NIOC with version %s" % self.version

            # Check resource state and apply all required changes
            if self.check_resources() == 'update':
                self.set_nioc_resources(self.resource_changes)

    def state_update_nioc_version(self):
        self.result['changed'] = True
        if not self.module.check_mode:
            self.set_nioc_version()
            self.result['dvswitch_nioc_status'] = "Set NIOC to version %s" % self.version

            # Check resource state and apply all required changes
            if self.check_resources() == 'update':
                self.set_nioc_resources(self.resource_changes)

    def state_update_nioc_resources(self):
        self.result['changed'] = True
        if not self.module.check_mode:
            self.result['dvswitch_nioc_status'] = "Resource configuration modified"
            self.set_nioc_resources(self.resource_changes)

    def set_nioc_enabled(self, state):
        try:
            self.dvs.EnableNetworkResourceManagement(enable=state)
        except vim.fault.DvsFault as dvs_fault:
            self.module.fail_json(msg='DvsFault while setting NIOC enabled=%r: %s' % (state, to_native(dvs_fault.msg)))
        except vim.fault.DvsNotAuthorized as auth_fault:
            self.module.fail_json(msg='Not authorized to set NIOC enabled=%r: %s' % (state, to_native(auth_fault.msg)))
        except vmodl.fault.NotSupported as support_fault:
            self.module.fail_json(msg='NIOC not supported by DVS: %s' % to_native(support_fault.msg))
        except vmodl.RuntimeFault as runtime_fault:
            self.module.fail_json(msg='RuntimeFault while setting NIOC enabled=%r: %s' % (state, to_native(runtime_fault.msg)))

    def set_nioc_version(self):
        upgrade_spec = vim.DistributedVirtualSwitch.ConfigSpec()
        upgrade_spec.configVersion = self.dvs.config.configVersion
        if not self.version:
            self.version = 'version2'
        upgrade_spec.networkResourceControlVersion = self.version

        try:
            task = self.dvs.ReconfigureDvs_Task(spec=upgrade_spec)
            wait_for_task(task)
        except vmodl.RuntimeFault as runtime_fault:
            self.module.fail_json(msg="RuntimeFault when setting NIOC version: %s " % to_native(runtime_fault.msg))

    def check_nioc_state(self):
        self.dvs = find_dvs_by_name(self.content, self.switch)

        if self.dvs is None:
            self.module.fail_json(msg='DVS %s was not found.' % self.switch)
        else:
            if not self.dvs.config.networkResourceManagementEnabled:
                return 'absent'
            if self.version and self.dvs.config.networkResourceControlVersion != self.version:
                return 'version'

            # NIOC is enabled and the correct version, so return the state of the resources
            return self.check_resources()

    def check_resources(self):
        self.dvs = find_dvs_by_name(self.content, self.switch)
        if self.dvs is None:
            self.module.fail_json(msg="DVS named '%s' was not found" % self.switch)

        for resource in self.resources:
            if self.check_resource_state(resource) == 'update':
                self.resource_changes.append(resource)
                self.result['resources_changed'].append(resource['name'])

        if len(self.resource_changes) > 0:
            return 'update'
        return 'present'

    def check_resource_state(self, resource):
        resource_cfg = self.find_netioc_by_key(resource['name'])
        if resource_cfg is None:
            self.module.fail_json(msg="NetIOC resource named '%s' was not found" % resource['name'])

        rc = {
            "limit": resource_cfg.allocationInfo.limit,
            "shares_level": resource_cfg.allocationInfo.shares.level
        }
        if resource_cfg.allocationInfo.shares.level == 'custom':
            rc["shares"] = resource_cfg.allocationInfo.shares.shares
        if self.dvs.config.networkResourceControlVersion == "version3":
            rc["reservation"] = resource_cfg.allocationInfo.reservation

        for k, v in rc.items():
            if k in resource and v != resource[k]:
                return 'update'
        return 'valid'

    def set_nioc_resources(self, resources):
        if self.dvs.config.networkResourceControlVersion == 'version3':
            self._update_version3_resources(resources)
        elif self.dvs.config.networkResourceControlVersion == 'version2':
            self._update_version2_resources(resources)

    def _update_version3_resources(self, resources):
        allocations = list()

        for resource in resources:
            allocation = vim.DistributedVirtualSwitch.HostInfrastructureTrafficResource()
            allocation.allocationInfo = vim.DistributedVirtualSwitch.HostInfrastructureTrafficResource.ResourceAllocation()
            allocation.key = resource['name']
            if 'limit' in resource:
                allocation.allocationInfo.limit = resource['limit']
            if 'reservation' in resource:
                allocation.allocationInfo.reservation = resource['reservation']
            if 'shares_level' in resource:
                allocation.allocationInfo.shares = vim.SharesInfo()
                allocation.allocationInfo.shares.level = resource['shares_level']
                if 'shares' in resource and resource['shares_level'] == 'custom':
                    allocation.allocationInfo.shares.shares = resource['shares']
                elif resource['shares_level'] == 'custom':
                    self.module.fail_json(
                        msg="Resource %s, shares_level set to custom but shares not specified" % resource['name']
                    )

            allocations.append(allocation)

        spec = vim.DistributedVirtualSwitch.ConfigSpec()
        spec.configVersion = self.dvs.config.configVersion
        spec.infrastructureTrafficResourceConfig = allocations

        task = self.dvs.ReconfigureDvs_Task(spec)
        wait_for_task(task)

    def _update_version2_resources(self, resources):
        allocations = list()

        for resource in resources:
            resource_cfg = self.find_netioc_by_key(resource['name'])
            allocation = vim.DVSNetworkResourcePoolConfigSpec()
            allocation.allocationInfo = vim.DVSNetworkResourcePoolAllocationInfo()
            allocation.key = resource['name']
            allocation.configVersion = resource_cfg.configVersion
            if 'limit' in resource:
                allocation.allocationInfo.limit = resource['limit']
            if 'shares_level' in resource:
                allocation.allocationInfo.shares = vim.SharesInfo()
                allocation.allocationInfo.shares.level = resource['shares_level']
                if 'shares' in resource and resource['shares_level'] == 'custom':
                    allocation.allocationInfo.shares.shares = resource['shares']

            allocations.append(allocation)

        self.dvs.UpdateNetworkResourcePool(allocations)

    def find_netioc_by_key(self, resource_name):
        config = None
        if self.dvs.config.networkResourceControlVersion == "version3":
            config = self.dvs.config.infrastructureTrafficResourceConfig
        elif self.dvs.config.networkResourceControlVersion == "version2":
            config = self.dvs.networkResourcePool

        for obj in config:
            if obj.key == resource_name:
                return obj
        return None


def main():
    argument_spec = vmware_argument_spec()

    argument_spec.update(
        dict(
            switch=dict(required=True, type='str', aliases=['dvswitch']),
            version=dict(type='str', choices=['version2', 'version3']),
            state=dict(default='present', choices=['present', 'absent'], type='str'),
            resources=dict(
                type='list',
                default=list(),
                elements='dict',
                options=dict(
                    name=dict(
                        type='str',
                        required=True,
                        choices=[
                            'faultTolerance',
                            'hbr',
                            'iSCSI',
                            'management',
                            'nfs',
                            'vdp',
                            'virtualMachine',
                            'vmotion',
                            'vsan'
                        ]
                    ),
                    limit=dict(type='int', default=-1),
                    shares_level=dict(
                        type='str',
                        required=False,
                        choices=[
                            'low',
                            'normal',
                            'high',
                            'custom'
                        ]
                    ),
                    shares=dict(type='int', required=False),
                    reservation=dict(type='int', default=0)
                )
            ),
        )
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    try:
        vmware_dvswitch_nioc = VMwareDVSwitchNIOC(module)
        vmware_dvswitch_nioc.process_state()
    except vmodl.RuntimeFault as runtime_fault:
        module.fail_json(msg=to_native(runtime_fault.msg))
    except vmodl.MethodFault as method_fault:
        module.fail_json(msg=to_native(method_fault.msg))
    except Exception as e:
        module.fail_json(msg=to_native(e))


if __name__ == '__main__':
    main()



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_ptp_interface
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxPtpInterface(TestOnyxModule):

    module = onyx_ptp_interface
    enabled = False
    interfaces = {'Eth1/1': ('ethernet', '1/1'), 'Vlan 1': ('vlan', '1')}

    def setUp(self):
        self.enabled = False
        super(TestOnyxPtpInterface, self).setUp()
        self.mock_get_config = patch.object(
            onyx_ptp_interface.OnyxPtpInterfaceModule, "_show_ptp_interface_config")
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestOnyxPtpInterface, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        config_file = 'onyx_show_ptp_interface.cfg'
        data = None
        if self.enabled:
            data = load_fixture(config_file)

        self.get_config.return_value = data
        self.load_config.return_value = None

    def test_ptp_disabled_no_change(self):
        for interface in self.interfaces:
            set_module_args(dict(state='disabled', name=interface))
            self.execute_module(changed=False)

    def test_ptp_disabled_with_change(self):
        self.enabled = True
        for interface in self.interfaces:
            set_module_args(dict(state='disabled', name=interface))
            interface_type, interface_id = self.interfaces.get(interface)
            commands = ['no interface %s %s ptp enable' % (interface_type, interface_id)]
            self.execute_module(changed=True, commands=commands)

    def test_ptp_enabled_no_change(self):
        self.enabled = True
        for interface in self.interfaces:
            set_module_args(dict(state='enabled', name=interface))
            self.execute_module(changed=False)

    def test_ptp_enabled_with_change(self):
        for interface in self.interfaces:
            set_module_args(dict(state='disabled', name=interface))
            interface_type, interface_id = self.interfaces.get(interface)
            set_module_args(dict(state='enabled', name=interface))
            commands = ['interface %s %s ptp enable' % (interface_type, interface_id)]
            self.execute_module(changed=True, commands=commands)

    def test_ptp_attributs_no_change(self):
        self.enabled = True
        for interface in self.interfaces:
            set_module_args(dict(state='enabled', name=interface, delay_request=0,
                                 announce_interval=-2, announce_timeout=3,
                                 sync_interval=-3))
        self.execute_module(changed=False)

    def test_ptp_attributs_with_change(self):
        self.enabled = True
        for interface in self.interfaces:
            set_module_args(dict(state='enabled', name=interface, delay_request=2,
                                 announce_interval=-1, announce_timeout=5, sync_interval=-1))
            interface_type, interface_id = self.interfaces.get(interface)
            commands = ['interface %s %s ptp delay-req interval 2' % (interface_type, interface_id),
                        'interface %s %s ptp announce interval -1' % (interface_type, interface_id),
                        'interface %s %s ptp announce timeout 5' % (interface_type, interface_id),
                        'interface %s %s ptp sync interval -1' % (interface_type, interface_id)]
        self.execute_module(changed=True, commands=commands)



#!/usr/bin/python
#
# @author: Gaurav Rastogi (grastogi@avinetworks.com)
#          Eric Anderson (eanderson@avinetworks.com)
# module_check: supported
#
# Copyright: (c) 2017 Gaurav Rastogi, <grastogi@avinetworks.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

DOCUMENTATION = '''
---
module: avi_microservicegroup
author: Gaurav Rastogi (@grastogi23) <grastogi@avinetworks.com>

short_description: Module for setup of MicroServiceGroup Avi RESTful Object
description:
    - This module is used to configure MicroServiceGroup object
    - more examples at U(https://github.com/avinetworks/devops)
requirements: [ avisdk ]
options:
    state:
        description:
            - The state that should be applied on the entity.
        default: present
        choices: ["absent", "present"]
    avi_api_update_method:
        description:
            - Default method for object update is HTTP PUT.
            - Setting to patch will override that behavior to use HTTP PATCH.
        default: put
        choices: ["put", "patch"]
    avi_api_patch_op:
        description:
            - Patch operation to use when using avi_api_update_method as patch.
        choices: ["add", "replace", "delete"]
    created_by:
        description:
            - Creator name.
    description:
        description:
            - User defined description for the object.
    name:
        description:
            - Name of the microservice group.
        required: true
    service_refs:
        description:
            - Configure microservice(es).
            - It is a reference to an object of type microservice.
    tenant_ref:
        description:
            - It is a reference to an object of type tenant.
    url:
        description:
            - Avi controller URL of the object.
    uuid:
        description:
            - Uuid of the microservice group.
extends_documentation_fragment:
- community.network.avi

'''

EXAMPLES = """
  - name: Create a Microservice Group that can be used for setting up Network security policy
    avi_microservicegroup:
      controller: '{{ controller }}'
      username: '{{ username }}'
      password: '{{ password }}'
      description: Group created by my Secure My App UI.
      name: vs-msg-marketing
      tenant_ref: admin
"""

RETURN = '''
obj:
    description: MicroServiceGroup (api/microservicegroup) object
    returned: success, changed
    type: dict
'''

from ansible.module_utils.basic import AnsibleModule
try:
    from ansible_collections.community.network.plugins.module_utils.network.avi.avi import (
        avi_common_argument_spec, avi_ansible_api, HAS_AVI)
except ImportError:
    HAS_AVI = False


def main():
    argument_specs = dict(
        state=dict(default='present',
                   choices=['absent', 'present']),
        avi_api_update_method=dict(default='put',
                                   choices=['put', 'patch']),
        avi_api_patch_op=dict(choices=['add', 'replace', 'delete']),
        created_by=dict(type='str',),
        description=dict(type='str',),
        name=dict(type='str', required=True),
        service_refs=dict(type='list',),
        tenant_ref=dict(type='str',),
        url=dict(type='str',),
        uuid=dict(type='str',),
    )
    argument_specs.update(avi_common_argument_spec())
    module = AnsibleModule(
        argument_spec=argument_specs, supports_check_mode=True)
    if not HAS_AVI:
        return module.fail_json(msg=(
            'Avi python API SDK (avisdk>=17.1) or requests is not installed. '
            'For more details visit https://github.com/avinetworks/sdk.'))
    return avi_ansible_api(module, 'microservicegroup',
                           set([]))


if __name__ == '__main__':
    main()



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_ntp
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxNTP(TestOnyxModule):

    module = onyx_ntp
    enabled = False

    def setUp(self):
        self.enabled = False
        super(TestOnyxNTP, self).setUp()
        self.mock_get_config = patch.object(
            onyx_ntp.OnyxNTPModule, "_show_ntp_config")
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestOnyxNTP, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        config_file = 'onyx_ntp_show.cfg'
        data = load_fixture(config_file)
        self.get_config.return_value = data
        self.load_config.return_value = None

    def test_ntp_state_no_change(self):
        set_module_args(dict(state='enabled'))
        self.execute_module(changed=False)

    def test_ntp_state_with_change(self):
        set_module_args(dict(state='disabled'))
        commands = ['no ntp enable']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_authenticate_state_no_change(self):
        set_module_args(dict(authenticate_state='disabled'))
        self.execute_module(changed=False)

    def test_ntp_authenticate_state_with_change(self):
        set_module_args(dict(authenticate_state='enabled'))
        commands = ['ntp authenticate']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_authentication_key_no_change(self):
        set_module_args(dict(ntp_authentication_keys=[dict(auth_key_id='22',
                                                           auth_key_encrypt_type='sha1',
                                                           auth_key_password='12345')]))
        self.execute_module(changed=False)

    def test_ntp_authentication_key_with_change(self):
        set_module_args(dict(ntp_authentication_keys=[dict(auth_key_id='22',
                                                           auth_key_encrypt_type='md5',
                                                           auth_key_password='12345')]))
        commands = ['ntp authentication-key 22 md5 12345']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_trusted_keys_with_change(self):
        set_module_args(dict(trusted_keys='22'))
        commands = ['ntp trusted-key 22']
        self.execute_module(changed=True, commands=commands)



#!/usr/bin/python
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: onyx_username
version_added: '0.2.0'
author: "Anas Shami (@anass)"
short_description: Configure username module
description:
  - This module provides declarative management of users/roles
    on Mellanox ONYX network devices.
notes:
options:
    username:
        description:
            - Create/Edit user using username
        type: str
        required: True
    full_name:
        description:
            - Set the full name of this user
        type: str
    nopassword:
        description:
            - Clear password for such user
        type: bool
        default: False
    password:
        description:
            - Set password fot such user
        type: str
    encrypted_password:
        description:
            - Decide the type of setted password (plain text or encrypted)
        type: bool
        default: False
    capability:
        description:
            - Grant capability to this user account
        type: str
        choices: ['monitor', 'unpriv', 'v_admin', 'admin']
    reset_capability:
        description:
            - Reset capability to this user account
        type: bool
        default: False
    disconnected:
        description:
            - Disconnect all sessions of this user
        type: bool
        default: False
    disabled:
        description:
            - Disable means of logging into this account
        type: str
        choices: ['none', 'login', 'password', 'all']
    state:
        description:
            - Set state of the given account
        default: present
        type: str
        choices: ['present', 'absent']
'''

EXAMPLES = """
- name: Create new user
  onyx_username:
      username: anass

- name: Set the user full-name
  onyx_username:
      username: anass
      full_name: anasshami

- name: Set the user encrypted password
  onyx_username:
      username: anass
      password: 12345
      encrypted_password: True

- name: Set the user capability
  onyx_username:
      username: anass
      capability: monitor

- name: Reset the user capability
  onyx_username:
      username: anass
      reset_capability: True

- name: Remove the user configuration
  onyx_username:
      username: anass
      state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device.
  returned: always
  type: list
  sample:
    - username *
    - username * password *
    - username * nopassword
    - username * disable login
    - username * capability admin
    - no username *
    - no username * disable

"""
import re

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.onyx.onyx import BaseOnyxModule, show_cmd


class OnyxUsernameModule(BaseOnyxModule):
    ACCOUNT_STATE = {
        'Account locked out': dict(disabled='all'),
        'No password required for login': dict(nopassword=True),
        'Local password login disabled': dict(disabled='password'),
        'Account disabled': dict(disabled='all')
    }
    ENCRYPTED_ID = 7

    def init_module(self):
        """
        module initialization
        """
        element_spec = dict()

        argument_spec = dict(state=dict(choices=['absent', 'present'], default='present'),
                             username=dict(type='str', required=True),
                             disabled=dict(choices=['none', 'login', 'password', 'all']),
                             capability=dict(choices=['monitor', 'unpriv', 'v_admin', 'admin']),
                             nopassword=dict(type='bool', default=False),
                             password=dict(type='str', no_log=True),
                             encrypted_password=dict(type='bool', default=False),
                             reset_capability=dict(type="bool", default=False),
                             disconnected=dict(type='bool', default=False),
                             full_name=dict(type='str'))
        argument_spec.update(element_spec)
        self._module = AnsibleModule(
            argument_spec=argument_spec,
            supports_check_mode=True,
            mutually_exclusive=[['password', 'nopassword']])

    def get_required_config(self):
        self._required_config = dict()
        module_params = self._module.params
        params = {}
        ''' Requred/Default fields '''
        params['username'] = module_params.get('username')
        params['state'] = module_params.get('state')
        params['encrypted_password'] = module_params.get('encrypted_password')
        params['reset_capability'] = module_params.get('reset_capability')
        ''' Other fields '''
        for key, value in module_params.items():
            if value is not None:
                params[key] = value
        self.validate_param_values(params)
        self._required_config = params

    def _get_username_config(self):
        return show_cmd(self._module, "show usernames", json_fmt=True, fail_on_error=False)

    def _set_current_config(self, users_config):
        '''
        users_config ex:
        {
            admin": [
                    {
                        "CAPABILITY": "admin",
                        "ACCOUNT STATUS": "No password required for login",
                        "FULL NAME": "System Administrator"
                    }
            ],
        }
        '''
        if not users_config:
            return
        current_config = self._current_config
        for username, config in users_config.items():
            config_json = config[0]
            current_config[username] = current_config.get(username, {})
            account_status = config_json.get('ACCOUNT STATUS')
            status_value = self.ACCOUNT_STATE.get(account_status)

            if status_value is not None:
                #  None for enabled account with password account "Password set (SHA512 | MD5)" so we won't change any attribute here.
                current_config[username].update(status_value)
            current_config[username].update({
                'capability': config_json.get('CAPABILITY'),
                'full_name': config_json.get('FULL NAME')
            })

    def load_current_config(self):
        self._current_config = dict()
        users_config = self._get_username_config()
        self._set_current_config(users_config)

    def generate_commands(self):
        current_config, required_config = self._current_config, self._required_config
        username = required_config.get('username')
        current_user = current_config.get(username)
        if current_user is not None:
            '''created account we just need to edit his attributes'''
            full_name = required_config.get('full_name')
            if full_name is not None and current_user.get('full_name') != full_name:
                self._commands.append("username {0} full-name {1}".format(username, full_name))

            disabled = required_config.get('disabled')
            if disabled is not None and current_user.get('disabled') != disabled:
                if disabled == 'none':
                    self._commands.append("no username {0} disable".format(username))
                elif disabled == 'all':
                    self._commands.append("username {0} disable".format(username))
                else:
                    self._commands.append("username {0} disable {1}".format(username, disabled))

            state = required_config.get('state')
            if state == 'absent':  # this will remove the user
                self._commands.append("no username {0}".format(username))

            capability = required_config.get('capability')
            if capability is not None and current_user.get('capability') != capability:
                self._commands.append("username {0} capability {1}".format(username, capability))

            reset_capability = required_config.get('reset_capability')
            if reset_capability:
                self._commands.append("no username {0} capability".format(username))

            password = required_config.get('password')
            if password is not None:
                encrypted = required_config.get('encrypted_password')
                if encrypted:
                    self._commands.append("username {0} password {1} {2}".format(username, self.ENCRYPTED_ID, password))
                else:
                    self._commands.append("username {0} password {1}".format(username, password))

            nopassword = required_config.get('nopassword')
            if nopassword and nopassword != current_user.get('nopassword', False):
                self._commands.append("username {0} nopassword".format(username))

            disconnected = required_config.get('disconnected')
            if disconnected:
                self._commands.append("username {0} disconnect".format(username))
        else:
            '''create new account if we have valid inforamtion, just check for username, capability, full_name, password'''

            capability = required_config.get('capability')
            password = required_config.get('password')
            full_name = required_config.get('full_name')
            if capability is not None or password is not None or full_name is not None:
                if capability is not None:
                    self._commands.append("username {0} capability {1}".format(username, capability))

                if password is not None:
                    encrypted = required_config.get('encrypted_password')
                    if encrypted:
                        self._commands.append("username {0} password {1} {2} ".format(username, self.ENCRYPTED_ID, password))
                    else:
                        self._commands.append("username {0} password {1}".format(username, password))

                if full_name is not None:
                    self._commands.append("username {0} full-name {1}".format(username, full_name))

            else:
                self._commands.append("username {0}".format(username))


def main():
    """ main entry point for module execution
    """
    OnyxUsernameModule.main()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: onyx_ntp
version_added: '0.2.0'
author: "Sara-Touqan (@sarato)"
short_description: Manage NTP general configurations and ntp keys configurations on Mellanox ONYX network devices
description:
  - This module provides declarative management of NTP & NTP Keys
    on Mellanox ONYX network devices.
options:
  state:
    description:
      - State of the NTP configuration.
    choices: ['enabled', 'disabled']
    type: str
  authenticate_state:
    description:
      - State of the NTP authentication configuration.
    choices: ['enabled', 'disabled']
    type: str
  ntp_authentication_keys:
    type: list
    description:
      - List of ntp authentication keys
    suboptions:
      auth_key_id:
        description:
          - Configures ntp key-id, range 1-65534
        required: true
        type: int
      auth_key_encrypt_type:
        description:
          - encryption type used to configure ntp authentication key.
        required: true
        choices: ['md5', 'sha1']
        type: str
      auth_key_password:
        description:
          - password used for ntp authentication key.
        required: true
        type: str
      auth_key_state:
         description:
             - Used to decide if you want to delete given ntp key or not
         choices: ['present', 'absent']
         type: str
  trusted_keys:
    type: list
    description:
      - List of ntp trusted keys
'''

EXAMPLES = """
- name: Configure NTP
  onyx_ntp:
    state: enabled
    authenticate_state: enabled
    ntp_authentication_keys:
            - auth_key_id: 1
              auth_key_encrypt_type: md5
              auth_key_password: 12345
              auth_key_state: absent
    trusted_keys: 1,2,3
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always.
  type: list
  sample:
    - ntp enable
    - ntp disable
    - ntp authenticate
    - no ntp authenticate
    - ntp authentication-key 1 md5 12345
    - no ntp authentication-key 1
    - ntp trusted-key 1,2,3
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.network.plugins.module_utils.network.onyx.onyx import BaseOnyxModule
from ansible_collections.community.network.plugins.module_utils.network.onyx.onyx import show_cmd


class OnyxNTPModule(BaseOnyxModule):

    def init_module(self):
        """ module initialization
        """
        ntp_authentication_key_spec = dict(auth_key_id=dict(type='int', required=True),
                                           auth_key_encrypt_type=dict(required=True, choices=['md5', 'sha1']),
                                           auth_key_password=dict(required=True),
                                           auth_key_state=dict(choices=['present', 'absent']))
        element_spec = dict(
            state=dict(choices=['enabled', 'disabled']),
            authenticate_state=dict(choices=['enabled', 'disabled']),
            ntp_authentication_keys=dict(type='list', elements='dict', options=ntp_authentication_key_spec),
            trusted_keys=dict(type='list', elements='int')
        )
        argument_spec = dict()
        argument_spec.update(element_spec)
        self._module = AnsibleModule(
            argument_spec=argument_spec,
            supports_check_mode=True)

    def _validate_key_id(self):
        keys_id_list = self._required_config.get("ntp_authentication_keys")
        if keys_id_list:
            for key_item in keys_id_list:
                key_id = key_item.get("auth_key_id")
                if (key_id < 1) or (key_id > 65534):
                    self._module.fail_json(
                        msg='Invalid Key value, value should be in the range 1-65534')

    def get_required_config(self):
        module_params = self._module.params
        self._required_config = dict(module_params)
        self.validate_param_values(self._required_config)
        self._validate_key_id()

    def _show_ntp_config(self):
        show_cmds = []
        cmd = "show ntp"
        show_cmds.append(show_cmd(self._module, cmd, json_fmt=True, fail_on_error=False))
        cmd = "show ntp keys"
        show_cmds.append(show_cmd(self._module, cmd, json_fmt=True, fail_on_error=False))
        return show_cmds

    def _set_ntp_keys_config(self, ntp_config):
        if not ntp_config:
            return
        for req_ntp_auth_key in ntp_config:
            ecryption_type = req_ntp_auth_key.get("Encryption Type")
            self._current_config[req_ntp_auth_key.get("header")] = ecryption_type

    def _set_ntp_config(self, ntp_config):
        ntp_config = ntp_config[0]
        if not ntp_config:
            return
        self._current_config['state'] = ntp_config.get("NTP is administratively")
        self._current_config['authenticate_state'] = ntp_config.get("NTP Authentication administratively")

    def load_current_config(self):
        self._current_config = dict()
        ntp_config = self._show_ntp_config()
        if ntp_config:
            if ntp_config[0]:
                self._set_ntp_config(ntp_config[0])
            if ntp_config[1]:
                self._set_ntp_keys_config(ntp_config[1])

    def generate_commands(self):
        current_state = self._current_config.get("state")
        state = self._required_config.get("state")
        if state is None:
            state = current_state
        if state is not None:
            if current_state != state:
                if state == 'enabled':
                    self._commands.append('ntp enable')
                else:
                    self._commands.append('no ntp enable')
        authenticate_state = self._required_config.get("authenticate_state")
        if authenticate_state:
            current_authenticate_state = self._current_config.get("authenticate_state")
            if authenticate_state is not None:
                if current_authenticate_state != authenticate_state:
                    if authenticate_state == 'enabled':
                        self._commands.append('ntp authenticate')
                    else:
                        self._commands.append('no ntp authenticate')
        req_ntp_auth_keys = self._required_config.get('ntp_authentication_keys')
        if req_ntp_auth_keys:
            if req_ntp_auth_keys is not None:
                for req_ntp_auth_key in req_ntp_auth_keys:
                    req_key_id = req_ntp_auth_key.get('auth_key_id')
                    req_key = 'NTP Key ' + str(req_key_id)
                    current_req_key = self._current_config.get(req_key)
                    auth_key_state = req_ntp_auth_key.get('auth_key_state')
                    req_encrypt_type = req_ntp_auth_key.get('auth_key_encrypt_type')
                    req_password = req_ntp_auth_key.get('auth_key_password')
                    if current_req_key:
                        if req_encrypt_type == current_req_key:
                            if auth_key_state:
                                if auth_key_state == 'absent':
                                    self._commands.append('no ntp authentication-key {0}' .format(req_key_id))
                            else:
                                continue
                        else:
                            if auth_key_state:
                                if auth_key_state == 'present':
                                    self._commands.append('ntp authentication-key {0} {1} {2}'
                                                          .format(req_key_id,
                                                                  req_encrypt_type,
                                                                  req_password))
                            else:
                                self._commands.append('ntp authentication-key {0} {1} {2}'
                                                      .format(req_key_id,
                                                              req_encrypt_type,
                                                              req_password))

                    else:
                        if auth_key_state:
                            if auth_key_state == 'present':
                                self._commands.append('ntp authentication-key {0} {1} {2}'
                                                      .format(req_key_id,
                                                              req_encrypt_type,
                                                              req_password))
                        else:
                            self._commands.append('ntp authentication-key {0} {1} {2}'
                                                  .format(req_key_id,
                                                          req_encrypt_type,
                                                          req_password))

        req_trusted_keys = self._required_config.get('trusted_keys')
        if req_trusted_keys:
            for key in req_trusted_keys:
                self._commands.append('ntp trusted-key {0}' .format(key))


def main():
    """ main entry point for module execution
    """
    OnyxNTPModule.main()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Andrew Klychkov (@Andersson007) <aaklychkov@mail.ru>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mysql_query
short_description: Run MySQL queries
description:
- Runs arbitrary MySQL queries.
- Pay attention, the module does not support check mode!
  All queries will be executed in autocommit mode.
version_added: '0.2.0'
options:
  query:
    description:
    - SQL query to run. Multiple queries can be passed using YAML list syntax.
    type: list
    elements: str
    required: yes
  positional_args:
    description:
    - List of values to be passed as positional arguments to the query.
    - Mutually exclusive with I(named_args).
    type: list
  named_args:
    description:
    - Dictionary of key-value arguments to pass to the query.
    - Mutually exclusive with I(positional_args).
    type: dict
  login_db:
    description:
    - Name of database to connect to and run queries against.
    type: str
  single_transaction:
    description:
    - Where passed queries run in a single transaction (C(yes)) or commit them one-by-one (C(no)).
    type: bool
    default: no
notes:
- To pass a query containing commas, use YAML list notation with hyphen (see EXAMPLES block).
author:
- Andrew Klychkov (@Andersson007)
extends_documentation_fragment:
- community.general.mysql

'''

EXAMPLES = r'''
- name: Simple select query to acme db
  mysql_query:
    login_db: acme
    query: SELECT * FROM orders

- name: Select query to db acme with positional arguments
  mysql_query:
    login_db: acme
    query: SELECT * FROM acme WHERE id = %s AND story = %s
    positional_args:
    - 1
    - test

- name: Select query to test_db with named_args
  mysql_query:
    login_db: test_db
    query: SELECT * FROM test WHERE id = %(id_val)s AND story = %(story_val)s
    named_args:
      id_val: 1
      story_val: test

- name: Run several insert queries against db test_db in single transaction
  mysql_query:
    login_db: test_db
    query:
    - INSERT INTO articles (id, story) VALUES (2, 'my_long_story')
    - INSERT INTO prices (id, price) VALUES (123, '100.00')
    single_transaction: yes
'''

RETURN = r'''
executed_queries:
    description: List of executed queries.
    returned: always
    type: list
    sample: ['SELECT * FROM bar', 'UPDATE bar SET id = 1 WHERE id = 2']
query_result:
    description:
    - List of lists (sublist for each query) containing dictionaries
      in column:value form representing returned rows.
    returned: changed
    type: list
    sample: [[{"Column": "Value1"},{"Column": "Value2"}], [{"ID": 1}, {"ID": 2}]]
rowcount:
    description: Number of affected rows for each subquery.
    returned: changed
    type: list
    sample: [5, 1]
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.mysql import (
    mysql_connect,
    mysql_common_argument_spec,
    mysql_driver,
    mysql_driver_fail_msg,
)
from ansible.module_utils._text import to_native

DML_QUERY_KEYWORDS = ('INSERT', 'UPDATE', 'DELETE')
# TRUNCATE is not DDL query but it also returns 0 rows affected:
DDL_QUERY_KEYWORDS = ('CREATE', 'DROP', 'ALTER', 'RENAME', 'TRUNCATE')


# ===========================================
# Module execution.
#

def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        query=dict(type='list', elements='str', required=True),
        login_db=dict(type='str'),
        positional_args=dict(type='list'),
        named_args=dict(type='dict'),
        single_transaction=dict(type='bool', default=False),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=(
            ('positional_args', 'named_args'),
        ),
    )

    db = module.params['login_db']
    connect_timeout = module.params['connect_timeout']
    login_user = module.params['login_user']
    login_password = module.params['login_password']
    ssl_cert = module.params['client_cert']
    ssl_key = module.params['client_key']
    ssl_ca = module.params['ca_cert']
    config_file = module.params['config_file']
    query = module.params["query"]
    if module.params["single_transaction"]:
        autocommit = False
    else:
        autocommit = True
    # Prepare args:
    if module.params.get("positional_args"):
        arguments = module.params["positional_args"]
    elif module.params.get("named_args"):
        arguments = module.params["named_args"]
    else:
        arguments = None

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    # Connect to DB:
    try:
        cursor, db_connection = mysql_connect(module, login_user, login_password,
                                              config_file, ssl_cert, ssl_key, ssl_ca, db,
                                              connect_timeout=connect_timeout,
                                              cursor_class='DictCursor', autocommit=autocommit)
    except Exception as e:
        module.fail_json(msg="unable to connect to database, check login_user and "
                             "login_password are correct or %s has the credentials. "
                             "Exception message: %s" % (config_file, to_native(e)))
    # Set defaults:
    changed = False

    max_keyword_len = len(max(DML_QUERY_KEYWORDS + DDL_QUERY_KEYWORDS, key=len))

    # Execute query:
    query_result = []
    executed_queries = []
    rowcount = []
    for q in query:
        try:
            cursor.execute(q, arguments)

        except Exception as e:
            if not autocommit:
                db_connection.rollback()

            cursor.close()
            module.fail_json(msg="Cannot execute SQL '%s' args [%s]: %s" % (q, arguments, to_native(e)))

        try:
            query_result.append([dict(row) for row in cursor.fetchall()])

        except Exception as e:
            if not autocommit:
                db_connection.rollback()

            module.fail_json(msg="Cannot fetch rows from cursor: %s" % to_native(e))

        # Check DML or DDL keywords in query and set changed accordingly:
        q = q.lstrip()[0:max_keyword_len].upper()
        for keyword in DML_QUERY_KEYWORDS:
            if keyword in q and cursor.rowcount > 0:
                changed = True

        for keyword in DDL_QUERY_KEYWORDS:
            if keyword in q:
                changed = True

        executed_queries.append(cursor._last_executed)
        rowcount.append(cursor.rowcount)

    # When the module run with the single_transaction == True:
    if not autocommit:
        db_connection.commit()

    # Create dict with returned values:
    kw = {
        'changed': changed,
        'executed_queries': executed_queries,
        'query_result': query_result,
        'rowcount': rowcount,
    }

    # Exit:
    module.exit_json(**kw)


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The exos lldp_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""
from __future__ import absolute_import, division, print_function
__metaclass__ = type

import re
from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils
from ansible_collections.community.network.plugins.module_utils.network.exos.argspec.lldp_interfaces.lldp_interfaces import Lldp_interfacesArgs
from ansible_collections.community.network.plugins.module_utils.network.exos.exos import send_requests


class Lldp_interfacesFacts(object):
    """ The exos lldp_interfaces fact class
    """

    def __init__(self, module, subspec='config', options='options'):
        self._module = module
        self.argument_spec = Lldp_interfacesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def populate_facts(self, connection, ansible_facts, data=None):
        """ Populate the facts for lldp_interfaces
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """

        if not data:
            request = [{
                "path": "/rest/restconf/data/openconfig-lldp:lldp/interfaces?depth=4",
                "method": "GET"
            }]
            data = send_requests(self._module, requests=request)

        objs = []
        if data:
            for d in data[0]["openconfig-lldp:interfaces"]["interface"]:
                obj = self.render_config(self.generated_spec, d["config"])
                if obj:
                    objs.append(obj)

        ansible_facts['ansible_network_resources'].pop('lldp_interfaces', None)
        facts = {}
        if objs:
            params = utils.validate_config(self.argument_spec, {'config': objs})
            facts['lldp_interfaces'] = params['config']

        ansible_facts['ansible_network_resources'].update(facts)
        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)

        config["name"] = conf["name"]
        config["enabled"] = bool(conf["enabled"])

        return utils.remove_empties(config)



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################
"""
The arg spec for the nxos_static_routes module
"""
from __future__ import absolute_import, division, print_function

__metaclass__ = type


class Static_routesArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_static_routes module
    """

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "elements": "dict",
            "options": {
                "address_families": {
                    "elements": "dict",
                    "options": {
                        "afi": {
                            "choices": ["ipv4", "ipv6"],
                            "required": True,
                            "type": "str",
                        },
                        "routes": {
                            "elements": "dict",
                            "options": {
                                "dest": {"required": True, "type": "str"},
                                "next_hops": {
                                    "elements": "dict",
                                    "options": {
                                        "admin_distance": {"type": "int"},
                                        "dest_vrf": {"type": "str"},
                                        "forward_router_address": {
                                            "type": "str"
                                        },
                                        "interface": {"type": "str"},
                                        "route_name": {"type": "str"},
                                        "tag": {"type": "int"},
                                        "track": {"type": "int"},
                                    },
                                    "type": "list",
                                },
                            },
                            "type": "list",
                        },
                    },
                    "type": "list",
                },
                "vrf": {"type": "str"},
            },
            "type": "list",
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "deleted",
                "merged",
                "overridden",
                "replaced",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: onyx_snmp_hosts
version_added: '0.2.0'
author: "Sara Touqan (@sarato)"
short_description: Configures SNMP host parameters
description:
  - This module provides declarative management of SNMP hosts protocol params
    on Mellanox ONYX network devices.
options:
  hosts:
    type: list
    description:
      - List of snmp hosts
    suboptions:
      name:
        description:
          - Specifies the name of the host.
        required: true
        type: str
      enabled:
        description:
          - Temporarily Enables/Disables sending of all notifications to this host.
        type: bool
      notification_type:
        description:
          - Configures the type of sending notification to the specified host.
        choices: ['trap', 'inform']
        type: str
      port:
        description:
             - Overrides default target port for this host.
        type: str
      version:
        description:
             -  Specifys SNMP version of informs to send.
        choices: ['1', '2c', '3']
        type: str
      user_name:
        description:
             - Specifys username for this inform sink.
        type: str
      auth_type:
        description:
             - Configures SNMP v3 security parameters, specifying passwords in a nother parameter (auth_password) (passwords are always stored encrypted).
        choices: ['md5', 'sha', 'sha224', 'sha256', 'sha384', 'sha512']
        type: str
      auth_password:
        description:
             - The password needed to configure the auth type.
        type: str
      privacy_type:
        description:
             -  Specifys SNMP v3 privacy settings for this user.
        choices: ['3des', 'aes-128', 'aes-192', 'aes-192-cfb', 'aes-256', 'aes-256-cfb', 'des']
        type: str
      privacy_password:
        description:
             - The password needed to configure the privacy type.
        type: str
      state:
        description:
             - Used to decide if you want to delete the specified host or not.
        choices: ['present' , 'absent']
        type: str
'''

EXAMPLES = """
- name: Enables snmp host
  onyx_snmp_hosts:
      hosts:
       - name: 1.1.1.1
         enabled: true

- name: Configures snmp host with version 2c
  onyx_snmp_hosts:
      hosts:
         - name: 2.3.2.4
           enabled: true
           notification_type: trap
           port: 66
           version: 2c

- name: Configures snmp host with version 3 and configures it with user as sara
  onyx_snmp_hosts:
       hosts:
         - name: 2.3.2.4
           enabled: true
           notification_type: trap
           port: 66
           version: 3
           user_name: sara
           auth_type: sha
           auth_password: jnbdfijbdsf
           privacy_type: 3des
           privacy_password: nojfd8uherwiugfh

- name: Deletes the snmp host
  onyx_snmp_hosts:
        hosts:
          - name: 2.3.2.4
            state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device.
  returned: always
  type: list
  sample:
    - snmp-server host <host_name> disable
    - no snmp-server host <host_name> disable
    - snmp-server host <host_name> informs port <port_number> version <version_number>
    - snmp-server host <host_name> traps port <port_number> version <version_number>
    - snmp-server host <host_name> informs port <port_number> version <version_number>  user <user_name> auth <auth_type>
                                                <auth_password> priv <privacy_type> <privacy_password>
    - snmp-server host <host_name> traps port <port_number> version <version_number>  user <user_name> auth <auth_type>
                                              <auth_password> priv <privacy_type> <privacy_password>
    - no snmp-server host <host_name>.
"""

import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.network.plugins.module_utils.network.onyx.onyx import show_cmd
from ansible_collections.community.network.plugins.module_utils.network.onyx.onyx import BaseOnyxModule


class OnyxSNMPHostsModule(BaseOnyxModule):

    def init_module(self):
        """ initialize module
        """
        host_spec = dict(name=dict(required=True),
                         enabled=dict(type='bool'),
                         notification_type=dict(type='str', choices=['trap', 'inform']),
                         port=dict(type='str'),
                         version=dict(type='str', choices=['1', '2c', '3']),
                         user_name=dict(type='str'),
                         auth_type=dict(type='str', choices=['md5', 'sha', 'sha224', 'sha256', 'sha384', 'sha512']),
                         privacy_type=dict(type='str', choices=['3des', 'aes-128', 'aes-192', 'aes-192-cfb', 'aes-256', 'aes-256-cfb', 'des']),
                         privacy_password=dict(type='str', no_log=True),
                         auth_password=dict(type='str', no_log=True),
                         state=dict(type='str', choices=['present', 'absent'])
                         )
        element_spec = dict(
            hosts=dict(type='list', elements='dict', options=host_spec),
        )
        argument_spec = dict()
        argument_spec.update(element_spec)
        self._module = AnsibleModule(
            argument_spec=argument_spec,
            supports_check_mode=True)

    def validate_snmp_required_params(self):
        req_hosts = self._required_config.get("hosts")
        if req_hosts:
            for host in req_hosts:
                version = host.get('version')
                if version:
                    if version == '3':
                        if host.get('user_name') is None or host.get('auth_type') is None or host.get('auth_password') is None:
                            self._module.fail_json(msg='user_name, auth_type and auth_password are required when version number is 3.')

                if host.get('notification_type') is not None:
                    if host.get('version') is None or host.get('port') is None:
                        self._module.fail_json(msg='port and version are required when notification_type is provided.')

                if host.get('auth_type') is not None:
                    if host.get('auth_password') is None:
                        self._module.fail_json(msg='auth_password is required when auth_type is provided.')

                if host.get('privacy_type') is not None:
                    if host.get('privacy_password') is None:
                        self._module.fail_json(msg='privacy_password is required when privacy_type is provided.')

    def get_required_config(self):
        module_params = self._module.params
        self._required_config = dict(module_params)
        self.validate_param_values(self._required_config)
        self.validate_snmp_required_params()

    def _set_host_config(self, hosts_config):
        hosts = hosts_config.get('Notification sinks')
        if hosts[0].get('Lines'):
            self._current_config['current_hosts'] = dict()
            self._current_config['host_names'] = []
            return

        current_hosts = dict()
        host_names = []
        for host in hosts:
            host_info = dict()
            for host_name in host:
                host_names.append(host_name)
                enabled = True
                first_entry = host.get(host_name)[0]
                if first_entry:
                    if first_entry.get('Enabled') == 'no':
                        enabled = False
                    notification_type = first_entry.get('Notification type')
                    notification_type = notification_type.split()
                    host_info['notification_type'] = notification_type[2]
                    version = notification_type[1][1:]
                    host_info['port'] = first_entry.get('Port')
                    host_info['name'] = host_name
                    host_info['enabled'] = enabled
                    host_info['version'] = version
                    if first_entry.get('Community') is None:
                        if len(first_entry) == 8:
                            host_info['user_name'] = first_entry.get('Username')
                            host_info['auth_type'] = first_entry.get('Authentication type')
                            host_info['privacy_type'] = first_entry.get('Privacy type')
                        elif len(host.get(host_name)) == 2:
                            second_entry = host.get(host_name)[1]
                            host_info['user_name'] = second_entry.get('Username')
                            host_info['auth_type'] = second_entry.get('Authentication type')
                            host_info['privacy_type'] = second_entry.get('Privacy type')
                        else:
                            host_info['user_name'] = ''
                            host_info['auth_type'] = ''
                            host_info['privacy_type'] = ''
                    else:
                        host_info['user_name'] = ''
                        host_info['auth_type'] = ''
                        host_info['privacy_type'] = ''
                current_hosts[host_name] = host_info
        self._current_config['current_hosts'] = current_hosts
        self._current_config['host_names'] = host_names

    def _show_hosts_config(self):
        cmd = "show snmp host"
        return show_cmd(self._module, cmd, json_fmt=True, fail_on_error=False)

    def load_current_config(self):
        self._current_config = dict()
        hosts_config = self._show_hosts_config()
        if hosts_config[1]:
            self._set_host_config(hosts_config[1])

    def generate_snmp_commands_with_current_config(self, host):
        host_id = host.get('name')
        host_notification_type = host.get('notification_type')
        host_enabled = host.get("enabled")
        host_port = host.get('port')
        host_version = host.get('version')
        host_username = host.get('user_name')
        host_auth_type = host.get('auth_type')
        host_auth_pass = host.get('auth_password')
        host_priv_type = host.get('privacy_type')
        host_priv_pass = host.get('privacy_password')
        present_state = host.get('state')
        current_hosts = self._current_config.get("current_hosts")
        current_entry = current_hosts.get(host_id)
        if present_state is not None:
            if present_state == 'absent':
                self._commands.append('no snmp-server host {0}' .format(host_id))
                return
        if host_enabled is not None:
            if current_entry.get('enabled') != host_enabled:
                if host_enabled is True:
                    self._commands.append('no snmp-server host {0} disable' .format(host_id))
                else:
                    self._commands.append('snmp-server host {0} disable' .format(host_id))
        if host_notification_type is not None:
            current_port = current_entry.get('port')
            current_version = current_entry.get('version')
            current_priv_type = current_entry.get('privacy_type')
            current_username = current_entry.get('user_name')
            current_auth_type = current_entry.get('auth_type')
            current_noti_type = current_entry.get('notification_type')
            if host_port is not None:
                if host_version is not None:
                    if host_version == '3':
                        if (host_priv_type is not None and host_priv_pass is not None):
                            if((current_noti_type != host_notification_type) or
                               ((current_port != host_port)) or
                               (current_version != host_version) or
                               (current_priv_type != host_priv_type) or
                               (current_username != host_username) or
                               (current_auth_type != host_auth_type)):
                                self._commands.append('snmp-server host {0} {1}s port {2} version {3} user {4} auth {5} {6} priv {7} {8}'
                                                      .format(host_id, host_notification_type, host_port,
                                                              host_version, host_username, host_auth_type, host_auth_pass,
                                                              host_priv_type, host_priv_pass))
                        else:
                            if((current_noti_type != host_notification_type) or
                               ((current_port != host_port)) or
                               (current_version != host_version) or
                               (current_username != host_username) or
                               (current_auth_type != host_auth_type)):
                                self._commands.append('snmp-server host {0} {1}s port {2} version {3} user {4} auth {5} {6}'
                                                      .format(host_id, host_notification_type,
                                                              host_port, host_version, host_username,
                                                              host_auth_type, host_auth_pass))
                    else:
                        if((current_noti_type != host_notification_type) or
                           ((current_port != host_port)) or
                           (current_version != host_version)):
                            self._commands.append('snmp-server host {0} {1}s port {2} version {3}'
                                                  .format(host_id, host_notification_type,
                                                          host_port, host_version))
                else:
                    if ((current_noti_type != host_notification_type) or
                       ((current_port != host_port))):
                        self._commands.append('snmp-server host {0} {1}s port {2}'
                                              .format(host_id, host_notification_type, host_port))
            else:
                if host_version is not None:
                    if host_version == '3':
                        if (host_priv_type is not None and host_priv_pass is not None):
                            if ((current_noti_type != host_notification_type) or
                               ((current_version != host_version)) or
                               (current_username != host_username) or
                               ((current_auth_type != host_auth_type)) or
                               (current_priv_type != host_priv_type)):
                                self._commands.append('snmp-server host {0} {1}s version {2} user {3} auth {4} {5} priv {6} {7}'
                                                      .format(host_id, host_notification_type, host_version, host_username,
                                                              host_auth_type, host_auth_pass, host_priv_type, host_priv_pass))

                        else:
                            if ((current_noti_type != host_notification_type) or
                               ((current_version != host_version)) or
                               (current_username != host_username) or
                               ((current_auth_type != host_auth_type))):
                                self._commands.append('snmp-server host {0} {1}s version {2} user {3} auth {4} {5}'
                                                      .format(host_id, host_notification_type,
                                                              host_version, host_username, host_auth_type, host_auth_pass))

                    else:
                        if ((current_noti_type != host_notification_type) or
                           ((current_version != host_version))):
                            self._commands.append('snmp-server host {0} {1}s version {2}' .format(host_id,
                                                  host_notification_type, host_version))

    def generate_snmp_commands_without_current_config(self, host):
        host_id = host.get('name')
        host_notification_type = host.get('notification_type')
        host_enabled = host.get("enabled")
        host_port = host.get('port')
        host_version = host.get('version')
        host_username = host.get('user_name')
        host_auth_type = host.get('auth_type')
        host_auth_pass = host.get('auth_password')
        host_priv_type = host.get('privacy_type')
        host_priv_pass = host.get('privacy_password')
        present_state = host.get('state')
        present_state = host.get('state')
        if present_state is not None:
            if present_state == 'absent':
                return
        if host_enabled is not None:
            if host_enabled is True:
                self._commands.append('no snmp-server host {0} disable' .format(host_id))
            else:
                self._commands.append('snmp-server host {0} disable' .format(host_id))

        if host_notification_type is not None:
            if host_port is not None:
                if host_version is not None:
                    if host_version == '3':
                        if (host_priv_type is not None and host_priv_pass is not None):
                            self._commands.append('snmp-server host {0} {1}s port {2} version {3} user {4} auth {5} {6} priv {7} {8}'
                                                  .format(host_id, host_notification_type, host_port, host_version, host_username,
                                                          host_auth_type, host_auth_pass, host_priv_type, host_priv_pass))
                        else:
                            self._commands.append('snmp-server host {0} {1}s port {2} version {3} user {4} auth {5} {6}'
                                                  .format(host_id, host_notification_type, host_port, host_version, host_username,
                                                          host_auth_type, host_auth_pass))
                    else:
                        self._commands.append('snmp-server host {0} {1}s port {2} version {3}' .format(host_id,
                                              host_notification_type, host_port, host_version))
                else:
                    self._commands.append('snmp-server host {0} {1}s port {2}' .format(host_id,
                                                                                       host_notification_type, host_port))
            else:
                if host_version is not None:
                    if host_version == '3':
                        if (host_priv_type is not None and host_priv_pass is not None):
                            self._commands.append('snmp-server host {0} {1}s version {2} user {3} auth {4} {5} priv {6} {7}'
                                                  .format(host_id, host_notification_type, host_version, host_username,
                                                          host_auth_type, host_auth_pass, host_priv_type, host_priv_pass))
                        else:
                            self._commands.append('snmp-server host {0} {1}s version {2} user {3} auth {4} {5}' .format(host_id,
                                                  host_notification_type, host_version, host_username,
                                                  host_auth_type, host_auth_pass))
                    else:
                        self._commands.append('snmp-server host {0} {1}s version {2}' .format(host_id,
                                              host_notification_type, host_version))

    def generate_commands(self):
        req_hosts = self._required_config.get("hosts")
        host_names = self._current_config.get("host_names")

        if req_hosts:
            for host in req_hosts:
                host_id = host.get('name')
                if host_id:
                    if host_names and (host_id in host_names):
                        self.generate_snmp_commands_with_current_config(host)
                    else:
                        self.generate_snmp_commands_without_current_config(host)


def main():
    """ main entry point for module execution
    """
    OnyxSNMPHostsModule.main()


if __name__ == '__main__':
    main()



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_vxlan
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxVxlanModule(TestOnyxModule):

    module = onyx_vxlan
    arp_suppression = True

    def setUp(self):
        super(TestOnyxVxlanModule, self).setUp()
        self.mock_get_vxlan_config = patch.object(
            onyx_vxlan.OnyxVxlanModule, "_show_vxlan_config")
        self.get_vxlan_config = self.mock_get_vxlan_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

        self.mock_get_nve_detail = patch.object(
            onyx_vxlan.OnyxVxlanModule, "_show_nve_detail")
        self.get_nve_detail = self.mock_get_nve_detail.start()

    def tearDown(self):
        super(TestOnyxVxlanModule, self).tearDown()
        self.mock_get_vxlan_config.stop()
        self.mock_load_config.stop()
        self.mock_get_nve_detail.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        interfaces_nve_config_file = 'onyx_show_interfaces_nve.cfg'
        interfaces_nve_detail_config_file = 'onyx_show_interfaces_nve_detail.cfg'
        self.get_nve_detail.return_value = None
        interfaces_nve_detail_data = load_fixture(interfaces_nve_detail_config_file)
        interfaces_nv_data = load_fixture(interfaces_nve_config_file)
        self.get_nve_detail.return_value = interfaces_nve_detail_data
        if self.arp_suppression is False:
            interfaces_nve_detail_data[0]["10"][0]["Neigh Suppression"] = "Disable"
            interfaces_nve_detail_data[0]["6"][0]["Neigh Suppression"] = "Disable"
            self.get_nve_detail.return_value = interfaces_nve_detail_data
        self.get_vxlan_config.return_value = interfaces_nv_data

        self.load_config.return_value = None

    def test_configure_vxlan_no_change(self):
        set_module_args(dict(nve_id=1, loopback_id=1, bgp=True, mlag_tunnel_ip='192.10.10.1',
                             vni_vlan_list=[dict(vlan_id=10, vni_id=10010), dict(vlan_id=6, vni_id=10060)],
                             arp_suppression=True))
        self.execute_module(changed=False)

    def test_configure_vxlan_with_change(self):
        set_module_args(dict(nve_id=2, loopback_id=1, bgp=True, mlag_tunnel_ip='192.10.10.1',
                             vni_vlan_list=[dict(vlan_id=10, vni_id=10010), dict(vlan_id=6, vni_id=10060)],
                             arp_suppression=True))
        commands = [
            "no interface nve 1", "interface nve 2", "exit",
            "interface nve 2 vxlan source interface loopback 1 ",
            "interface nve 2 nve controller bgp", "interface nve 2 vxlan mlag-tunnel-ip 192.10.10.1",
            "interface nve 2 nve neigh-suppression", "interface nve 2 nve vni 10010 vlan 10",
            "interface vlan 10", "exit", "interface nve 2 nve vni 10060 vlan 6", "interface vlan 6", "exit"
        ]
        self.execute_module(changed=True, commands=commands)

    def test_loopback_id_with_change(self):
        set_module_args(dict(nve_id=1, loopback_id=2, bgp=True, mlag_tunnel_ip='192.10.10.1',
                             vni_vlan_list=[dict(vlan_id=10, vni_id=10010), dict(vlan_id=6, vni_id=10060)],
                             arp_suppression=True))
        commands = ["interface nve 1 vxlan source interface loopback 2 "]
        self.execute_module(changed=True, commands=commands)

    def test_mlag_tunnel_ip_with_change(self):
        set_module_args(dict(nve_id=1, loopback_id=1, bgp=True, mlag_tunnel_ip='192.10.10.10',
                             vni_vlan_list=[dict(vlan_id=10, vni_id=10010), dict(vlan_id=6, vni_id=10060)],
                             arp_suppression=True))
        commands = ["interface nve 1 vxlan mlag-tunnel-ip 192.10.10.10"]
        self.execute_module(changed=True, commands=commands)

    def test_vni_vlan_list_with_change(self):
        set_module_args(dict(nve_id=1, loopback_id=1, bgp=True, mlag_tunnel_ip='192.10.10.1',
                             vni_vlan_list=[dict(vlan_id=11, vni_id=10011), dict(vlan_id=7, vni_id=10061)],
                             arp_suppression=False))
        commands = ["interface nve 1 nve vni 10011 vlan 11", "interface nve 1 nve vni 10061 vlan 7"]
        self.execute_module(changed=True, commands=commands)

    def test_arp_suppression_with_change(self):
        self.arp_suppression = False
        set_module_args(dict(nve_id=1, loopback_id=1, bgp=True, mlag_tunnel_ip='192.10.10.1',
                             vni_vlan_list=[dict(vlan_id=10, vni_id=10010), dict(vlan_id=6, vni_id=10060)],
                             arp_suppression=True))
        commands = ["interface vlan 10", "exit", "interface vlan 6", "exit"]
        self.execute_module(changed=True, commands=commands)



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_bfd
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxBFDModule(TestOnyxModule):

    module = onyx_bfd

    def setUp(self):
        self.enabled = False
        super(TestOnyxBFDModule, self).setUp()
        self.mock_get_config = patch.object(
            onyx_bfd.OnyxBFDModule, "_show_bfd_config")
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestOnyxBFDModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        config_file = 'onyx_show_bfd.cfg'
        self.get_config.return_value = load_fixture(config_file)
        self.load_config.return_value = None

    def test_bfd_shutdown_no_change(self):
        set_module_args(dict(shutdown=True))
        self.execute_module(changed=False)

    def test_bfd_shutdown_with_change(self):
        set_module_args(dict(shutdown=False))
        commands = ['no ip bfd shutdown']
        self.execute_module(changed=True, commands=commands)

    def test_vrf_bfd_shutdown_no_change(self):
        set_module_args(dict(shutdown=False,
                             vrf='3'))
        self.execute_module(changed=False)

    def test_vrf_bfd_shutdown_with_change(self):
        set_module_args(dict(shutdown=True,
                             vrf='3'))
        commands = ['ip bfd shutdown vrf 3']
        self.execute_module(changed=True, commands=commands)

    def test_bfd_interval_no_change(self):
        set_module_args(dict(interval_min_rx=50,
                             interval_multiplier=7,
                             interval_transmit_rate=55))
        self.execute_module(changed=False)

    def test_bfd_interval_with_change(self):
        set_module_args(dict(interval_min_rx=55,
                             interval_multiplier=7,
                             interval_transmit_rate=100))
        commands = ['ip bfd interval min-rx 55 multiplier 7 transmit-rate 100 force']
        self.execute_module(changed=True, commands=commands)

    def test_vrf_bfd_interval_no_change(self):
        set_module_args(dict(interval_min_rx=50,
                             interval_multiplier=7,
                             interval_transmit_rate=55,
                             vrf='3'))
        self.execute_module(changed=False)

    def test_vrf_bfd_interval_with_change(self):
        set_module_args(dict(interval_min_rx=55,
                             interval_multiplier=7,
                             interval_transmit_rate=100,
                             vrf='3'))
        commands = ['ip bfd vrf 3 interval min-rx 55 multiplier 7 transmit-rate 100 force']
        self.execute_module(changed=True, commands=commands)

    def test_bfd_iproute_no_change(self):
        set_module_args(dict(iproute_network_prefix='1.1.1.0',
                             iproute_mask_length=24,
                             iproute_next_hop='3.2.2.2'))
        self.execute_module(changed=False)

    def test_bfd_iproute_with_change(self):
        set_module_args(dict(iproute_network_prefix='1.1.1.0',
                             iproute_mask_length=24,
                             iproute_next_hop='3.2.2.3'))
        commands = ['ip route 1.1.1.0 /24 3.2.2.3 bfd']
        self.execute_module(changed=True, commands=commands)

    def test_vrf_bfd_iproute_no_change(self):
        set_module_args(dict(iproute_network_prefix='1.1.1.0',
                             iproute_mask_length=24,
                             iproute_next_hop='3.2.2.2',
                             vrf='3'))
        self.execute_module(changed=False)

    def test_vrf_bfd_iproute_with_change(self):
        set_module_args(dict(iproute_network_prefix='1.1.1.0',
                             iproute_mask_length=24,
                             iproute_next_hop='3.2.2.3',
                             vrf='3'))
        commands = ['ip route vrf 3 1.1.1.0 /24 3.2.2.3 bfd']
        self.execute_module(changed=True, commands=commands)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Bruno Calogero <brunocalogero@hotmail.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: aci_switch_policy_leaf_profile
short_description: Manage switch policy leaf profiles (infra:NodeP)
description:
- Manage switch policy leaf profiles on Cisco ACI fabrics.
options:
  leaf_profile:
    description:
    - The name of the Leaf Profile.
    type: str
    aliases: [ leaf_profile_name, name ]
  description:
    description:
    - Description for the Leaf Profile.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci

seealso:
- module: aci_switch_policy_leaf_profile
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(infra:NodeP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Bruno Calogero (@brunocalogero)
'''

EXAMPLES = r'''
- name: creating a Leaf Profile with description
  cisco.aci.aci_switch_policy_leaf_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    leaf_profile: sw_name
    description: sw_description
    state: present
  delegate_to: localhost

- name: Deleting a Leaf Profile
  cisco.aci.aci_switch_policy_leaf_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    leaf_profile: sw_name
    state: absent
  delegate_to: localhost

- name: Query a Leaf Profile
  cisco.aci.aci_switch_policy_leaf_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    leaf_profile: sw_name
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(
        leaf_profile=dict(type='str', aliases=['name', 'leaf_profile_name']),  # Not required for querying all objects
        description=dict(type='str', aliases=['descr']),
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
        name_alias=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['leaf_profile']],
            ['state', 'present', ['leaf_profile']],
        ],
    )

    leaf_profile = module.params.get('leaf_profile')
    description = module.params.get('description')
    state = module.params.get('state')
    name_alias = module.params.get('name_alias')

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class='infraNodeP',
            aci_rn='infra/nprof-{0}'.format(leaf_profile),
            module_object=leaf_profile,
            target_filter={'name': leaf_profile},
        ),
    )

    aci.get_existing()

    if state == 'present':
        aci.payload(
            aci_class='infraNodeP',
            class_config=dict(
                name=leaf_profile,
                descr=description,
                nameAlias=name_alias,
            ),
        )

        aci.get_diff(aci_class='infraNodeP')

        aci.post_config()

    elif state == 'absent':
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: aci_aaa_user_certificate
short_description: Manage AAA user certificates (aaa:UserCert)
description:
- Manage AAA user certificates on Cisco ACI fabrics.
options:
  aaa_user:
    description:
    - The name of the user to add a certificate to.
    type: str
    required: yes
  aaa_user_type:
    description:
    - Whether this is a normal user or an appuser.
    type: str
    choices: [ appuser, user ]
    default: user
  certificate:
    description:
    - The PEM format public key extracted from the X.509 certificate.
    type: str
    aliases: [ cert_data, certificate_data ]
  certificate_name:
    description:
    - The name of the user certificate entry in ACI.
    type: str
    aliases: [ cert_name ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci

notes:
- The C(aaa_user) must exist before using this module in your playbook.
  The M(aci_aaa_user) module can be used for this.
seealso:
- module: aci_aaa_user
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(aaa:UserCert).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dag Wieers (@dagwieers)
'''

EXAMPLES = r'''
- name: Add a certificate to user
  cisco.aci.aci_aaa_user_certificate:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: admin
    certificate_name: admin
    certificate_data: '{{ lookup("file", "pki/admin.crt") }}'
    state: present
  delegate_to: localhost

- name: Remove a certificate of a user
  cisco.aci.aci_aaa_user_certificate:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: admin
    certificate_name: admin
    state: absent
  delegate_to: localhost

- name: Query a certificate of a user
  cisco.aci.aci_aaa_user_certificate:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: admin
    certificate_name: admin
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all certificates of a user
  cisco.aci.aci_aaa_user_certificate:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: admin
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec

ACI_MAPPING = dict(
    appuser=dict(
        aci_class='aaaAppUser',
        aci_mo='userext/appuser-',
    ),
    user=dict(
        aci_class='aaaUser',
        aci_mo='userext/user-',
    ),
)


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(
        aaa_user=dict(type='str', required=True),
        aaa_user_type=dict(type='str', default='user', choices=['appuser', 'user']),
        certificate=dict(type='str', aliases=['cert_data', 'certificate_data']),
        certificate_name=dict(type='str', aliases=['cert_name']),  # Not required for querying all objects
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
        name_alias=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['aaa_user', 'certificate_name']],
            ['state', 'present', ['aaa_user', 'certificate', 'certificate_name']],
        ],
    )

    aaa_user = module.params.get('aaa_user')
    aaa_user_type = module.params.get('aaa_user_type')
    certificate = module.params.get('certificate')
    certificate_name = module.params.get('certificate_name')
    state = module.params.get('state')
    name_alias = module.params.get('name_alias')

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class=ACI_MAPPING.get(aaa_user_type).get('aci_class'),
            aci_rn=ACI_MAPPING.get(aaa_user_type).get('aci_mo') + aaa_user,
            module_object=aaa_user,
            target_filter={'name': aaa_user},
        ),
        subclass_1=dict(
            aci_class='aaaUserCert',
            aci_rn='usercert-{0}'.format(certificate_name),
            module_object=certificate_name,
            target_filter={'name': certificate_name},
        ),
    )
    aci.get_existing()

    if state == 'present':
        aci.payload(
            aci_class='aaaUserCert',
            class_config=dict(
                data=certificate,
                name=certificate_name,
                nameAlias=name_alias,

            ),
        )

        aci.get_diff(aci_class='aaaUserCert')

        aci.post_config()

    elif state == 'absent':
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()
